Package: HCUPtools
Type: Package
Title: Access and Work with HCUP Resources and Datasets
Version: 1.0.1
Depends: R (>= 4.1.0)
Authors@R: person("Vikrant Dev Rathore", email = "rathore.vikrant@gmail.com", role = c("aut", "cre"))
Description: A comprehensive R package for accessing and working with publicly 
    available and free resources from the Agency for Healthcare Research and Quality 
    (AHRQ) Healthcare Cost and Utilization Project (HCUP). The package provides 
    streamlined access to HCUP's Clinical Classifications Software Refined (CCSR) 
    mapping files and Summary Trend Tables, enabling researchers and analysts to 
    efficiently map ICD-10-CM diagnosis codes and ICD-10-PCS procedure codes to 
    CCSR categories and access HCUP statistical reports. Key features include: 
    direct download from HCUP website, multiple output formats (long/wide/default), 
    cross-classification support, version management, citation generation, and 
    intelligent caching. The package does not redistribute HCUP data files but 
    facilitates direct download from the official HCUP website, ensuring users 
    always have access to the latest versions and maintain compliance with HCUP 
    data use policies. This package only accesses free public tools and reports; 
    it does NOT access HCUP databases (NIS, KID, SID, NEDS, etc.) that require 
    purchase. For more information, see <https://hcup-us.ahrq.gov/>.
License: MIT + file LICENSE
URL: https://github.com/vikrant31/HCUPtools,
        https://vikrant31.github.io/HCUPtools/
BugReports: https://github.com/vikrant31/HCUPtools/issues
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: httr2, readr, dplyr, tidyr, tibble, utils, stats, rlang, xml2,
        readxl
Suggests: testthat (>= 3.0.0), knitr, rmarkdown, data.table, pdftools
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-05-11 09:29:38 UTC; vikrant31
Author: Vikrant Dev Rathore [aut, cre]
Maintainer: Vikrant Dev Rathore <rathore.vikrant@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-11 15:20:02 UTC
Built: R 4.5.2; ; 2026-05-11 15:49:25 UTC; unix

Package: HCUPtools
Type: Package
Title: Access and Work with HCUP Resources and Datasets
Version: 1.0.1
Depends: R (>= 4.1.0)
Authors@R: person("Vikrant Dev Rathore", email = "rathore.vikrant@gmail.com", role = c("aut", "cre"))
Description: A comprehensive R package for accessing and working with publicly 
    available and free resources from the Agency for Healthcare Research and Quality 
    (AHRQ) Healthcare Cost and Utilization Project (HCUP). The package provides 
    streamlined access to HCUP's Clinical Classifications Software Refined (CCSR) 
    mapping files and Summary Trend Tables, enabling researchers and analysts to 
    efficiently map ICD-10-CM diagnosis codes and ICD-10-PCS procedure codes to 
    CCSR categories and access HCUP statistical reports. Key features include: 
    direct download from HCUP website, multiple output formats (long/wide/default), 
    cross-classification support, version management, citation generation, and 
    intelligent caching. The package does not redistribute HCUP data files but 
    facilitates direct download from the official HCUP website, ensuring users 
    always have access to the latest versions and maintain compliance with HCUP 
    data use policies. This package only accesses free public tools and reports; 
    it does NOT access HCUP databases (NIS, KID, SID, NEDS, etc.) that require 
    purchase. For more information, see <https://hcup-us.ahrq.gov/>.
License: MIT + file LICENSE
URL: https://github.com/vikrant31/HCUPtools,
        https://vikrant31.github.io/HCUPtools/
BugReports: https://github.com/vikrant31/HCUPtools/issues
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: httr2, readr, dplyr, tidyr, tibble, utils, stats, rlang, xml2,
        readxl
Suggests: testthat (>= 3.0.0), knitr, rmarkdown, data.table, pdftools
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-05-11 09:29:38 UTC; vikrant31
Author: Vikrant Dev Rathore [aut, cre]
Maintainer: Vikrant Dev Rathore <rathore.vikrant@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-11 15:20:02 UTC
Built: R 4.6.0; ; 2026-05-11 16:02:14 UTC; unix

# Generated by roxygen2: do not edit by hand

export(ccsr_changelog)
export(ccsr_map)
export(download_ccsr)
export(download_trend_tables)
export(get_ccsr_description)
export(hcup_citation)
export(list_ccsr_versions)
export(list_trend_table_sheets)
export(read_ccsr)
export(read_trend_table)
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_rows)
importFrom(dplyr,desc)
importFrom(dplyr,distinct)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,row_number)
importFrom(dplyr,select)
importFrom(dplyr,ungroup)
importFrom(httr2,req_perform)
importFrom(httr2,req_timeout)
importFrom(httr2,req_user_agent)
importFrom(httr2,request)
importFrom(httr2,resp_body_raw)
importFrom(httr2,resp_body_string)
importFrom(readr,cols)
importFrom(readr,locale)
importFrom(readr,read_csv)
importFrom(readxl,excel_sheets)
importFrom(readxl,read_excel)
importFrom(rlang,.data)
importFrom(stats,setNames)
importFrom(tibble,as_tibble)
importFrom(tibble,tibble)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,pivot_wider)
importFrom(utils,person)
importFrom(utils,unzip)
importFrom(xml2,read_html)
importFrom(xml2,xml_attr)
importFrom(xml2,xml_find_all)
importFrom(xml2,xml_text)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE,
  fig.width = 7,
  fig.height = 5
)

## ----eval = FALSE-------------------------------------------------------------
# # Install from CRAN (evaluated only when you knit locally; skipped during R CMD check)
# install.packages("HCUPtools")
# 
# # Load the package
# library(HCUPtools)
# library(dplyr)  # For data manipulation examples

## ----eval=FALSE---------------------------------------------------------------
# # Download the latest diagnosis CCSR mapping file
# dx_map <- download_ccsr("diagnosis")
# 
# # Download the latest procedure CCSR mapping file
# pr_map <- download_ccsr("procedure")

## ----eval=FALSE---------------------------------------------------------------
# # Download a specific version (useful for reproducibility)
# dx_map_v2025 <- download_ccsr("diagnosis", version = "v2025.1")
# pr_map_v2025 <- download_ccsr("procedure", version = "v2025.1")

## ----eval=FALSE---------------------------------------------------------------
# # List all available versions
# all_versions <- list_ccsr_versions()
# print(all_versions)
# 
# # List only diagnosis versions
# dx_versions <- list_ccsr_versions("diagnosis")
# 
# # List only procedure versions
# pr_versions <- list_ccsr_versions("procedure")

## ----eval=FALSE---------------------------------------------------------------
# # Create sample patient data with ICD-10 diagnosis codes
# patient_data <- tibble::tibble(
#   patient_id = 1:10,
#   admission_date = as.Date(c("2024-01-15", "2024-02-20", "2024-03-10",
#                               "2024-04-05", "2024-05-12", "2024-06-18",
#                               "2024-07-22", "2024-08-30", "2024-09-14",
#                               "2024-10-08")),
#   icd10_dx = c("E11.9", "I10", "M79.3", "E78.5", "K21.9",
#                "I50.9", "N18.6", "E78.5", "I25.10", "J44.1")
# )

## ----eval=FALSE---------------------------------------------------------------
# # Map codes using long format (default)
# mapped_long <- ccsr_map(
#   data = patient_data,
#   code_col = "icd10_dx",
#   map_df = dx_map,
#   output_format = "long"
# )
# 
# # View the results
# head(mapped_long, 20)
# 
# # Count occurrences of each CCSR category
# ccsr_counts <- mapped_long |>
#   count(ccsr_category, sort = TRUE)
# print(ccsr_counts)

## ----eval=FALSE---------------------------------------------------------------
# # Map codes using wide format
# mapped_wide <- ccsr_map(
#   data = patient_data,
#   code_col = "icd10_dx",
#   map_df = dx_map,
#   output_format = "wide"
# )
# 
# # View the results
# head(mapped_wide)

## ----eval=FALSE---------------------------------------------------------------
# # Map codes using default category only
# mapped_default <- ccsr_map(
#   data = patient_data,
#   code_col = "icd10_dx",
#   map_df = dx_map,
#   default_only = TRUE
# )
# 
# # View the results
# head(mapped_default)

## ----eval=FALSE---------------------------------------------------------------
# # Get descriptions for specific CCSR codes
# ccsr_codes <- c("ADM010", "NEP003", "CIR019", "END001", "MBD001")
# descriptions <- get_ccsr_description(ccsr_codes, map_df = dx_map)
# print(descriptions)
# 
# # Get descriptions without pre-downloaded mapping (will download automatically)
# descriptions_auto <- get_ccsr_description(
#   c("ADM010", "NEP003"),
#   type = "diagnosis"
# )

## ----eval=FALSE---------------------------------------------------------------
# # Download procedure mapping
# pr_map <- download_ccsr("procedure")
# 
# # Create sample procedure data
# procedure_data <- tibble::tibble(
#   case_id = 1:5,
#   procedure_date = as.Date(c("2024-01-20", "2024-02-15", "2024-03-22",
#                               "2024-04-10", "2024-05-18")),
#   icd10_pcs = c("0DB60ZZ", "0DT70ZZ", "0WQ3XZ", "0FB00ZZ", "0HB00ZX")
# )
# 
# # Map procedure codes
# mapped_procedures <- ccsr_map(
#   data = procedure_data,
#   code_col = "icd10_pcs",
#   map_df = pr_map
# )
# 
# # View the results
# head(mapped_procedures)

## ----eval=FALSE---------------------------------------------------------------
# # Step 1: Download mapping file
# dx_map <- download_ccsr("diagnosis")
# 
# # Step 2: Map diagnosis codes
# patient_data_mapped <- ccsr_map(
#   data = patient_data,
#   code_col = "icd10_dx",
#   map_df = dx_map,
#   output_format = "long"
# )
# 
# # Step 3: Count occurrences of each CCSR category
# ccsr_counts <- patient_data_mapped |>
#   count(ccsr_category, sort = TRUE)
# 
# # Step 4: Merge with descriptions for reporting
# ccsr_counts_with_desc <- ccsr_counts |>
#   left_join(
#     get_ccsr_description(
#       unique(patient_data_mapped$ccsr_category),
#       map_df = dx_map
#     ),
#     by = c("ccsr_category" = "ccsr_code")
#   )
# 
# # Step 5: View the final results
# print(ccsr_counts_with_desc)

## ----eval=FALSE---------------------------------------------------------------
# # List available tables (interactive menu)
# available_tables <- download_trend_tables()
# print(available_tables)
# 
# # Download a specific table by ID
# # Table 2a: All Inpatient Encounter Types - Trends in Number of Discharges
# table_path <- download_trend_tables("2a")
# 
# # Download all tables as a ZIP file (~81 MB)
# all_tables_zip <- download_trend_tables("all")

## ----eval=FALSE---------------------------------------------------------------
# # Read the trend table data
# trend_data <- read_trend_table(table_path, sheet = "National")
# head(trend_data)
# 
# # List available sheets
# sheets <- list_trend_table_sheets(table_path)
# print(sheets)
# 
# # Read specific state data
# california_data <- read_trend_table(table_path, sheet = "California")

## ----eval=FALSE---------------------------------------------------------------
# # Get change log as data table (default)
# changelog <- ccsr_changelog(version = "v2026.1")
# print(changelog)
# 
# # Get change log URL
# changelog_url <- ccsr_changelog(version = "v2026.1", format = "url")
# 
# # View change log in default PDF viewer
# ccsr_changelog(version = "v2026.1", format = "view")
# 
# # Download change log file
# changelog_file <- ccsr_changelog(version = "v2026.1", format = "download")

## ----eval=FALSE---------------------------------------------------------------
# # Generate text citation for CCSR
# cat(hcup_citation())
# 
# # Generate citation for Summary Trend Tables
# cat(hcup_citation(resource = "trend_tables"))
# 
# # Generate BibTeX citation (for LaTeX documents)
# cat(hcup_citation(format = "bibtex"))
# 
# # Generate R citation object (for R markdown)
# citation_obj <- hcup_citation(format = "r")
# print(citation_obj)

## ----eval=FALSE---------------------------------------------------------------
# # Read CCSR file from various formats
# dx_map <- read_ccsr("path/to/DXCCSR-v2026-1.zip")
# dx_map <- read_ccsr("path/to/DXCCSR_v2026-1.csv")
# dx_map <- read_ccsr("path/to/DXCCSR_v2026-1.xlsx")
# dx_map <- read_ccsr("path/to/extracted_directory/")
# 
# # Read trend table Excel file
# national_data <- read_trend_table(
#   "path/to/HCUP_SummaryTrendTables_T2a.xlsx",
#   sheet = "National"
# )

#' Download CCSR Mapping Files from HCUP
#'
#' Downloads and loads Clinical Classifications Software Refined (CCSR) mapping
#' files directly from the Agency for Healthcare Research and Quality (AHRQ)
#' Healthcare Cost and Utilization Project (HCUP) website.
#'
#' @param type Character string specifying the type of CCSR file to download.
#'   Must be one of: "diagnosis" (or "dx") for ICD-10-CM diagnosis codes, or
#'   "procedure" (or "pr") for ICD-10-PCS procedure codes. Default is "diagnosis".
#' @param version Character string specifying the CCSR version to download.
#'   Use "latest" to download the most recent version, or specify a version
#'   like "v2026.1", "v2025.1", etc. Default is "latest".
#' @param cache Logical. If TRUE (default), the downloaded file is cached in
#'   a temporary directory to avoid re-downloading on subsequent calls.
#' @param clean_names Logical. If TRUE (default), column names are cleaned to
#'   follow R naming conventions (snake_case).
#'
#' @return A tibble containing the CCSR mapping data with the following columns:
#'   - For diagnosis files: ICD-10-CM code, CCSR category, default CCSR category,
#'     and clinical descriptions
#'   - For procedure files: ICD-10-PCS code, CCSR category, and descriptions
#'
#' @details
#' This function downloads CCSR mapping files directly from the HCUP website.
#' The package does not redistribute these files but facilitates access to the
#' official AHRQ data sources.
#'
#' The function handles:
#' - Automatic URL construction based on type and version
#' - ZIP file download and extraction
#' - Proper encoding of special characters
#' - Preservation of leading zeros in ICD-10 codes
#' - Conversion to tidy tibble format
#'
#' @examples
#' \donttest{
#' # Download latest diagnosis CCSR mapping
#' dx_map <- download_ccsr("diagnosis")
#'
#' # Download specific version of procedure CCSR mapping
#' pr_map <- download_ccsr("procedure", version = "v2025.1")
#'
#' # Download without caching
#' dx_map <- download_ccsr("diagnosis", cache = FALSE)
#' }
#'
#' @importFrom httr2 request req_timeout req_user_agent req_perform resp_body_string resp_body_raw
#' @importFrom readr read_csv cols locale
#' @importFrom tibble as_tibble
#' @importFrom utils unzip
#' @importFrom xml2 read_html xml_find_all xml_attr xml_text
#' @export
download_ccsr <- function(type = "diagnosis",
                          version = "latest",
                          cache = TRUE,
                          clean_names = TRUE) {
  # Validate type
  type <- tolower(type)
  if (type %in% c("dx", "diagnosis")) {
    type <- "diagnosis"
  } else if (type %in% c("pr", "procedure")) {
    type <- "procedure"
  } else {
    stop("`type` must be one of: 'diagnosis'/'dx' or 'procedure'/'pr'")
  }

  # Get version information - do this early to ensure we have latest
  original_version <- version
  if (version == "latest") {
    # Always clear version cache when user explicitly requests "latest"
    # This ensures we get the freshest version from the website
    cache_name <- paste0("ccsr_latest_version_", type)
    cache_path <- file.path(tempdir(), cache_name)
    if (file.exists(cache_path)) {
      unlink(cache_path)  # Always clear to force fresh discovery
    }
    
    # Force refresh when requesting "latest" to ensure we get the actual latest
    version <- get_latest_version(type, force_refresh = TRUE)
    # Store the actual latest version for later comparison
    actual_latest_version <- version
  } else {
    actual_latest_version <- version
  }

  # Validate version format (accept both v2026.1 and v2026-1)
  if (!grepl("^v\\d{4}[-.]\\d+$", version)) {
    stop("`version` must be in format 'vYYYY.N' or 'vYYYY-N' (e.g., 'v2026.1' or 'v2026-1')")
  }

  # Convert version format to URL format (v2026.1 -> v2026-1)
  version_url <- gsub("\\.", "-", version)
  
  base_url <- "https://hcup-us.ahrq.gov/toolssoftware/ccsr/"
  
  # URL pattern varies by type and version:
  # - Recent diagnosis files (v2026+): DXCCSR-v2026-1.zip (hyphen)
  # - Older diagnosis files (v2025 and earlier): DXCCSR_v2025-1.zip (underscore)
  # - All procedure files: PRCCSR_v2026-1.zip (underscore)
  
  if (type == "diagnosis") {
    # Try hyphen first (newer versions), fall back to underscore (older versions)
    file_name_hyphen <- paste0("DXCCSR-", version_url, ".zip")
    file_name_underscore <- paste0("DXCCSR_", version_url, ".zip")
    
    # Check which pattern exists (try hyphen first for newer versions)
    url_hyphen <- paste0(base_url, file_name_hyphen)
    url_underscore <- paste0(base_url, file_name_underscore)
    
    # For v2026+, use hyphen; for older versions, use underscore
    year <- as.numeric(sub("v(\\d{4})[-.].*", "\\1", version))
    if (year >= 2026) {
      url <- url_hyphen
      file_name <- file_name_hyphen
    } else {
      url <- url_underscore
      file_name <- file_name_underscore
    }
  } else {
    # Procedure files always use underscore
    file_name <- paste0("PRCCSR_", version_url, ".zip")
    url <- paste0(base_url, file_name)
  }

  # Check cache if enabled
  if (cache) {
    cache_dir <- file.path(tempdir(), "HCUPtools_cache")
    if (!dir.exists(cache_dir)) {
      dir.create(cache_dir, recursive = TRUE)
    }
    cache_file <- file.path(cache_dir, file_name)
    
    # If version is "latest", check ALL cached files and find the best match
    if (original_version == "latest") {
      # Find all cached files for this type
      pattern <- if (type == "diagnosis") "DXCCSR.*\\.zip$" else "PRCCSR.*\\.zip$"
      all_cached <- list.files(cache_dir, pattern = pattern, full.names = TRUE, ignore.case = TRUE)
      
      if (length(all_cached) > 0) {
        # Extract versions from all cached files
        cached_versions <- sapply(all_cached, function(f) {
          version_match <- regmatches(basename(f), 
                                     regexpr("v\\d{4}[-.]\\d+", basename(f), 
                                            ignore.case = TRUE))
          if (length(version_match) > 0) {
            gsub("-", ".", version_match[1])
          } else {
            NA_character_
          }
        })
        
        # Find the cached file with the latest version
        valid_cached <- all_cached[!is.na(cached_versions)]
        valid_versions <- cached_versions[!is.na(cached_versions)]
        
        if (length(valid_cached) > 0) {
          # Compare with actual latest version
          actual_latest_normalized <- gsub("-", ".", actual_latest_version)
          
          # Check if any cached file matches the latest version
          matching_idx <- which(valid_versions == actual_latest_normalized)
          
          if (length(matching_idx) > 0) {
            # Found a cached file matching latest version
            cache_file <- valid_cached[matching_idx[1]]
            message("Using cached file: ", cache_file)
            zip_path <- cache_file
          } else {
            # No cached file matches latest version - download new one
            message("Cached files found (", paste(valid_versions, collapse = ", "), 
                   ") but latest is ", actual_latest_version, 
                   ". Downloading latest version...")
            # Remove old cached files (optional - could keep them)
            # unlink(valid_cached)
            zip_path <- download_file(url, cache_file)
          }
        } else {
          # Couldn't parse versions from cached files
          zip_path <- download_file(url, cache_file)
        }
      } else {
        # No cached files found
        zip_path <- download_file(url, cache_file)
      }
    } else {
      # Specific version requested - check if exact file exists
      if (file.exists(cache_file)) {
        message("Using cached file: ", cache_file)
        zip_path <- cache_file
      } else {
        zip_path <- download_file(url, cache_file)
      }
    }
  } else {
    zip_path <- download_file(url, tempfile(fileext = ".zip"))
  }

  mapping_data <- extract_and_read_ccsr(zip_path, type, version, clean_names)
  return(mapping_data)
}

#' Helper function to download file with error handling
#'
#' @noRd
download_file <- function(url, destfile) {
  message("Downloading from: ", url)
  
  tryCatch({
    resp <- httr2::request(url) |>
      httr2::req_timeout(60) |>
      httr2::req_user_agent("HCUPtools R package") |>
      httr2::req_perform()
    
    httr2::resp_body_raw(resp) |>
      writeBin(destfile)
    
    message("Download complete: ", destfile)
    return(destfile)
  }, error = function(e) {
    # Try alternative URL pattern if first attempt fails
    if (grepl("DXCCSR", url)) {
      alt_url <- gsub("DXCCSR-", "DXCCSR_", url)
      alt_url <- gsub("DXCCSR_", "DXCCSR-", alt_url)
      
      if (alt_url != url) {
        tryCatch({
          message("Trying alternative URL pattern: ", alt_url)
          resp <- httr2::request(alt_url) |>
            httr2::req_timeout(60) |>
            httr2::req_user_agent("HCUPtools R package") |>
            httr2::req_perform()
          
          httr2::resp_body_raw(resp) |>
            writeBin(destfile)
          
          message("Download complete: ", destfile)
          return(destfile)
        }, error = function(e2) {
          stop("Failed to download file from both URL patterns. ",
               "Original error: ", conditionMessage(e), ". ",
               "Alternative error: ", conditionMessage(e2))
        })
      }
    }
    stop("Failed to download file: ", conditionMessage(e))
  })
}

#' Extract and read CCSR mapping file from ZIP
#'
#' @noRd
extract_and_read_ccsr <- function(zip_path, type, version, clean_names) {
  # Create temporary extraction directory
  extract_dir <- tempfile("ccsr_extract_")
  dir.create(extract_dir, showWarnings = FALSE)
  on.exit(unlink(extract_dir, recursive = TRUE), add = TRUE)
  
  # Extract ZIP file
  utils::unzip(zip_path, exdir = extract_dir)
  
  # Find CSV or Excel file in extracted directory
  all_files <- list.files(extract_dir, recursive = TRUE, full.names = TRUE)
  
  # Look for CSV files first (preferred)
  csv_files <- all_files[grepl("\\.csv$", all_files, ignore.case = TRUE)]
  xlsx_files <- all_files[grepl("\\.xlsx?$", all_files, ignore.case = TRUE)]
  
  # Filter by type if possible
  if (type == "diagnosis") {
    csv_files <- csv_files[grepl("DX|diagnosis", basename(csv_files), ignore.case = TRUE)]
    xlsx_files <- xlsx_files[grepl("DX|diagnosis", basename(xlsx_files), ignore.case = TRUE)]
  } else {
    csv_files <- csv_files[grepl("PR|procedure", basename(csv_files), ignore.case = TRUE)]
    xlsx_files <- xlsx_files[grepl("PR|procedure", basename(xlsx_files), ignore.case = TRUE)]
  }
  
  # If no type-specific files found, use all CSV/Excel files
  if (length(csv_files) == 0 && length(xlsx_files) == 0) {
    csv_files <- all_files[grepl("\\.csv$", all_files, ignore.case = TRUE)]
    xlsx_files <- all_files[grepl("\\.xlsx?$", all_files, ignore.case = TRUE)]
  }
  
  mapping_file <- NULL
  if (length(csv_files) > 0) {
    mapping_file <- csv_files[1]
  } else if (length(xlsx_files) > 0) {
    if (!requireNamespace("readxl", quietly = TRUE)) {
      stop("Package 'readxl' is required for reading Excel files. ",
           "Install it with: install.packages('readxl')")
    }
    mapping_file <- xlsx_files[1]
  } else {
    stop("No CSV or Excel file found in the downloaded ZIP archive")
  }
  
  if (length(mapping_file) > 1) {
    warning("Multiple mapping files found, using: ", mapping_file[1])
    mapping_file <- mapping_file[1]
  }

  message("Reading mapping file: ", basename(mapping_file))
  
  mapping_data <- readr::read_csv(
    mapping_file,
    col_types = readr::cols(.default = "c"),
    locale = readr::locale(encoding = "UTF-8"),
    show_col_types = FALSE
  )

  mapping_data <- tibble::as_tibble(mapping_data)

  if (clean_names) {
    names(mapping_data) <- clean_column_names(names(mapping_data))
  }

  # Normalize quoted values found in HCUP files (e.g., 'DIG001')
  mapping_data <- normalize_ccsr_character_columns(mapping_data)

  icd_col <- find_icd_column(mapping_data, type)
  if (!is.null(icd_col)) {
    mapping_data[[icd_col]] <- format_icd_codes(mapping_data[[icd_col]])
  }

  return(mapping_data)
}

#' Helper function to clean column names
#'
#' @noRd
clean_column_names <- function(names) {
  names <- gsub("\\s+", "_", names)
  names <- gsub("[^A-Za-z0-9_]", "", names)
  names <- tolower(names)
  names <- gsub("_+", "_", names)
  names <- gsub("^_|_$", "", names)
  return(names)
}

#' Helper function to find ICD code column
#'
#' @noRd
find_icd_column <- function(df, type) {
  col_names <- tolower(names(df))
  original_names <- names(df)
  
  if (type == "diagnosis") {
    patterns <- c("icd.*10.*cm", "icd.*10", "diagnosis.*code", "^code$", "^dx$")
  } else {
    patterns <- c("icd.*10.*pcs", "icd.*10", "procedure.*code", "^code$", "^pr$")
  }
  
  for (pattern in patterns) {
    match_idx <- grep(pattern, col_names)
    if (length(match_idx) > 0) {
      return(original_names[match_idx[1]])
    }
  }
  
  return(original_names[1])
}

#' Get latest available CCSR version from HCUP website
#'
#' Fetches the latest available CCSR version from the HCUP website
#' for the specified type (diagnosis or procedure). Results are cached for
#' 6 hours to minimize website requests.
#'
#' @param type Character string, either "diagnosis" or "procedure"
#' @return Character string with version (e.g., "v2026.1")
#'
#' @noRd
get_latest_version <- function(type, force_refresh = FALSE) {
  # Cache the result to avoid repeated requests
  cache_name <- paste0("ccsr_latest_version_", type)
  cache_path <- file.path(tempdir(), cache_name)
  
  # If force_refresh is TRUE, clear cache and fetch fresh
  if (force_refresh && file.exists(cache_path)) {
    unlink(cache_path)
  }
  
  # Check cache (valid for 6 hours)
  if (!force_refresh && file.exists(cache_path)) {
    cache_info <- file.info(cache_path)
    cache_age <- as.numeric(Sys.time() - cache_info$mtime, units = "hours")
    if (cache_age < 6) {
      cached_version <- readLines(cache_path, n = 1, warn = FALSE)
      if (nchar(cached_version) > 0 && grepl("^v\\d{4}[-.]\\d+$", cached_version)) {
        # Verify cached version is not too old (should be within last 2 years)
        year_match <- regmatches(cached_version, regexec("v(\\d{4})", cached_version))[[1]]
        if (length(year_match) >= 2) {
          cached_year <- as.numeric(year_match[2])
          current_year <- as.numeric(format(Sys.Date(), "%Y"))
          # Accept cached version if it's from current year or next year
          if (cached_year >= current_year) {
            return(cached_version)
          }
        }
      }
    }
  }
  
  file_prefix <- if (type == "diagnosis") "DXCCSR" else "PRCCSR"
  
  latest_version <- tryCatch({
    fetch_hcup_versions(file_prefix)
  }, error = function(e) {
    warning("Could not fetch latest version from HCUP website: ", 
            conditionMessage(e), 
            ". Using fallback version based on current year")
    NULL
  })
  
  # Validate the version format
  if (is.null(latest_version) || !grepl("^v\\d{4}[-.]\\d+$", latest_version)) {
    # If we can't fetch, try direct file checking as last resort
    latest_version <- try_fetch_versions_direct(file_prefix)
    if (is.null(latest_version) || length(latest_version) == 0) {
      # Last resort: use current year + 1 as fallback (to catch early releases)
      current_year <- as.numeric(format(Sys.Date(), "%Y"))
      # Check if next year version exists, otherwise use current year
      test_year <- current_year + 1
      test_file <- if (file_prefix == "DXCCSR" && test_year >= 2026) {
        paste0(file_prefix, "-v", test_year, "-1.zip")
      } else {
        paste0(file_prefix, "_v", test_year, "-1.zip")
      }
      test_url <- paste0("https://hcup-us.ahrq.gov/toolssoftware/ccsr/", test_file)
      
      next_year_exists <- tryCatch({
        resp <- httr2::request(test_url) |>
          httr2::req_method("HEAD") |>
          httr2::req_user_agent("HCUPtools R package") |>
          httr2::req_timeout(3) |>
          httr2::req_perform()
        httr2::resp_status(resp) %in% c(200, 302, 301)
      }, error = function(e) FALSE)
      
      if (next_year_exists) {
        latest_version <- paste0("v", test_year, ".1")
      } else {
        latest_version <- paste0("v", current_year, ".1")
      }
      warning("Could not determine latest version from HCUP website. ",
              "Using fallback version: ", latest_version,
              ". This may not be the actual latest version.")
    } else {
      latest_version <- get_latest_from_versions(latest_version)
    }
  }
  
  writeLines(latest_version, cache_path)
  return(latest_version)
}

#' Fetch available CCSR versions from HCUP website
#'
#' @param file_prefix Character string, either "DXCCSR" or "PRCCSR"
#' @return Character string with latest version, or NULL if not found
#'
#' @noRd
fetch_hcup_versions <- function(file_prefix) {
  # Try direct file checking FIRST (most reliable)
  # This directly checks if files exist, which is more accurate than parsing HTML
  versions_direct <- try_fetch_versions_direct(file_prefix)
  if (!is.null(versions_direct) && length(versions_direct) > 0) {
    # Direct file check found versions - use these (most reliable)
    return(get_latest_from_versions(versions_direct))
  }
  
  # Fallback: Try multiple URLs to find versions
  urls_to_try <- c(
    "https://hcup-us.ahrq.gov/toolssoftware/ccsr/ccs_refined.jsp",
    "https://hcup-us.ahrq.gov/toolssoftware/ccsr/",
    "https://hcup-us.ahrq.gov/toolssoftware/ccsr/ccsr_archive.jsp"
  )
  
  all_versions <- character(0)
  
  for (url in urls_to_try) {
    versions <- try_fetch_versions_from_url(url, file_prefix)
    if (!is.null(versions) && length(versions) > 0) {
      all_versions <- c(all_versions, versions)
    }
  }
  
  if (length(all_versions) > 0) {
    # Remove duplicates and get latest
    all_versions <- unique(all_versions)
    return(get_latest_from_versions(all_versions))
  }
  
  NULL
}

#' Fetch versions from a specific URL
#'
#' @param url Character string, URL to fetch
#' @param file_prefix Character string, either "DXCCSR" or "PRCCSR"
#' @return Character vector of version strings, or NULL if error
#'
#' @noRd
try_fetch_versions_from_url <- function(url, file_prefix) {
  tryCatch({
    resp <- httr2::request(url) |>
      httr2::req_user_agent("HCUPtools R package") |>
      httr2::req_timeout(10) |>
      httr2::req_perform()
    
    html_content <- httr2::resp_body_string(resp)
    doc <- xml2::read_html(html_content)
    
    links <- xml2::xml_find_all(doc, "//a[@href]")
    hrefs <- xml2::xml_attr(links, "href")
    
    text_nodes <- xml2::xml_find_all(doc, "//text()[normalize-space()]")
    text_content <- xml2::xml_text(text_nodes)
    all_text <- paste(text_content, collapse = " ")
    
    all_content <- c(hrefs, all_text)
    
    # Pattern to match both v2026.1 and v2026-1 formats
    version_pattern <- paste0(file_prefix, ".*?(v\\d{4}[-.]\\d+)")
    version_matches <- regmatches(all_content, 
                                 gregexpr(version_pattern, all_content, 
                                         ignore.case = TRUE))
    versions <- unlist(version_matches)
    
    if (length(versions) > 0) {
      versions <- gsub(paste0(".*(v\\d{4}[-.]\\d+).*"), "\\1", versions, 
                      ignore.case = TRUE)
      versions <- unique(versions)
      # Normalize to dot format for internal use (v2026-1 -> v2026.1)
      versions <- gsub("-", ".", versions)
      versions <- grep("^v\\d{4}\\.\\d+$", versions, value = TRUE, ignore.case = TRUE)
      return(versions)
    }
    
    NULL
  }, error = function(e) {
    NULL
  })
}

#' Get the latest version from a vector of version strings
#'
#' @param versions Character vector of version strings (e.g., c("v2026.1", "v2025.1"))
#' @return Character string with the latest version
#'
#' @noRd
get_latest_from_versions <- function(versions) {
  if (length(versions) == 0) {
    return(NULL)
  }
  
  # Normalize all versions to vYYYY.N format
  normalized <- vapply(versions, function(v) {
    # Handle both v2026.1 and v2026-1 formats
    v_clean <- gsub("-", ".", v)
    parts <- regmatches(v_clean, regexec("v(\\d{4})\\.(\\d+)", v_clean, ignore.case = TRUE))[[1]]
    if (length(parts) >= 3) {
      year <- as.numeric(parts[2])
      minor <- as.numeric(parts[3])
      # Create sortable numeric value: year * 1000 + minor
      # This ensures v2026.1 > v2025.10
      sort_value <- year * 1000 + minor
      return(sort_value)
    } else {
      return(NA_real_)
    }
  }, numeric(1))
  
  valid_idx <- !is.na(normalized)
  if (any(valid_idx)) {
    # Find the maximum sort value
    max_idx <- which.max(normalized[valid_idx])
    latest_version <- versions[valid_idx][max_idx]
    # Normalize to dot format
    latest_version <- gsub("-", ".", latest_version)
    return(latest_version)
  }
  
  # Fallback: return first version
  return(gsub("-", ".", versions[1]))
}

#' Try to fetch versions by attempting direct file access
#'
#' @param file_prefix Character string, either "DXCCSR" or "PRCCSR"
#' @return Character vector of version strings, or NULL if error
#'
#' @noRd
try_fetch_versions_direct <- function(file_prefix) {
  # Try to find versions by checking common patterns
  # Check next year, current year, and previous year (to catch early releases)
  current_year <- as.numeric(format(Sys.Date(), "%Y"))
  versions_found <- character(0)
  
  # Determine correct separator based on file type and year
  # DXCCSR v2026+ uses hyphen, DXCCSR v2025 and earlier uses underscore
  # PRCCSR always uses underscore
  use_hyphen_for_year <- function(year) {
    file_prefix == "DXCCSR" && year >= 2026
  }
  
  # Check years in order: next year (2026), current year (2025), previous year (2024)
  # This catches early releases of next year's version
  years_to_check <- c(current_year + 1, current_year, current_year - 1)
  
  for (year in years_to_check) {
    # Check minor versions (usually 1-2, but check up to 3)
    for (minor in 1:3) {
      # Determine separator BEFORE version (not between year and minor)
      # Year and minor are always separated by hyphen: v2026-1
      # But the separator before "v" depends on file type and year
      prefix_sep <- if (use_hyphen_for_year(year)) "-" else "_"
      version_str <- paste0("v", year, "-", minor)  # Always hyphen between year and minor
      file_name <- paste0(file_prefix, prefix_sep, version_str, ".zip")
      url <- paste0("https://hcup-us.ahrq.gov/toolssoftware/ccsr/", file_name)
      
      # Quick HEAD request to check if file exists
      file_exists <- tryCatch({
        resp <- httr2::request(url) |>
          httr2::req_method("HEAD") |>
          httr2::req_user_agent("HCUPtools R package") |>
          httr2::req_timeout(5) |>
          httr2::req_perform()
        
        status <- httr2::resp_status(resp)
        status %in% c(200, 302, 301, 303)
      }, error = function(e) FALSE)
      
      if (file_exists) {
        versions_found <- c(versions_found, paste0("v", year, ".", minor))
        # If we found next year version, return immediately (it's the latest)
        if (year > current_year) {
          return(versions_found)  # Next year found - this is definitely the latest
        }
        # For current year or previous year, continue checking all minor versions
      }
      
      # If hyphen didn't work for DXCCSR v2026+, try underscore as fallback (only for first minor)
      if (!file_exists && use_hyphen_for_year(year) && minor == 1) {
        prefix_sep <- "_"
        version_str <- paste0("v", year, "-", minor)  # Always hyphen between year and minor
        file_name <- paste0(file_prefix, prefix_sep, version_str, ".zip")
        url <- paste0("https://hcup-us.ahrq.gov/toolssoftware/ccsr/", file_name)
        
        file_exists <- tryCatch({
          resp <- httr2::request(url) |>
            httr2::req_method("HEAD") |>
            httr2::req_user_agent("HCUPtools R package") |>
            httr2::req_timeout(5) |>
            httr2::req_perform()
          
          status <- httr2::resp_status(resp)
          status %in% c(200, 302, 301, 303)
        }, error = function(e) FALSE)
        
        if (file_exists) {
          versions_found <- c(versions_found, paste0("v", year, ".", minor))
          # If we found next year version, return immediately
          if (year > current_year) {
            return(versions_found)
          }
        }
      }
    }
    
    # After checking all minor versions for next year, if we found any, return them
    if (year > current_year && length(versions_found) > 0) {
      return(versions_found)
    }
  }
  
  if (length(versions_found) > 0) {
    return(unique(versions_found))
  }
  
  NULL
}

#' Helper function to format ICD codes (preserve leading zeros)
#'
#' @noRd
format_icd_codes <- function(codes) {
  # Remove surrounding quotes and whitespace, preserve leading zeros
  codes <- strip_surrounding_quotes(codes)
  codes <- trimws(as.character(codes))
  return(codes)
}

#' Strip surrounding quotes from character values
#'
#' @noRd
strip_surrounding_quotes <- function(x) {
  x <- as.character(x)
  x <- trimws(x)
  x <- gsub("^[\"'`]+|[\"'`]+$", "", x)
  x <- gsub("^[\u2018\u2019\u201C\u201D]+|[\u2018\u2019\u201C\u201D]+$", "", x)
  x
}

#' Normalize character columns in CCSR mapping data
#'
#' @noRd
normalize_ccsr_character_columns <- function(df) {
  chr_cols <- vapply(df, is.character, logical(1))
  if (any(chr_cols)) {
    df[chr_cols] <- lapply(df[chr_cols], strip_surrounding_quotes)
  }
  df
}

# Test file for utility functions

test_that("format_icd_codes preserves leading zeros", {
  codes <- c("E08.00", "001.0", "123")
  formatted <- format_icd_codes(codes)
  expect_type(formatted, "character")
  expect_equal(formatted[1], "E08.00")
})

test_that("format_icd_codes handles whitespace", {
  codes <- c("  E11.9  ", "I10", " M79.3 ")
  formatted <- format_icd_codes(codes)
  expect_equal(formatted[1], "E11.9")
  expect_equal(formatted[3], "M79.3")
})

test_that("format_icd_codes strips surrounding quotes", {
  codes <- c("'A000'", "\"B111\"")
  formatted <- format_icd_codes(codes)
  expect_equal(formatted, c("A000", "B111"))
})

test_that("get_ccsr_description matches quoted mapping codes", {
  map_df <- tibble::tibble(
    ccsr_category = c("'DIG001'", "'RSP002'"),
    ccsr_category_description = c("Digestive disorders", "Respiratory disorders")
  )

  result <- get_ccsr_description(c("DIG001", "RSP002"), map_df = map_df, type = "diagnosis")
  expect_equal(result$description, c("Digestive disorders", "Respiratory disorders"))
})

test_that("hcup_citation returns correct format", {
  text_cite <- hcup_citation(format = "text")
  expect_type(text_cite, "character")
  expect_true(nchar(text_cite) > 0)
  
  bibtex_cite <- hcup_citation(format = "bibtex")
  expect_type(bibtex_cite, "character")
  expect_true(grepl("@misc", bibtex_cite))
  
  r_cite <- hcup_citation(format = "r")
  expect_type(r_cite, "list")
})

# Test file for download_ccsr function
# Note: These tests may require internet connection and may be skipped in CI/CD

test_that("download_ccsr validates type argument", {
  expect_error(download_ccsr(type = "invalid"), 
               "must be one of")
  # Note: Actual download test skipped to avoid internet dependency in CRAN checks
  skip_on_cran()
})

test_that("download_ccsr validates version argument", {
  expect_error(download_ccsr(type = "diagnosis", version = "invalid"),
               "must be in format")
})

test_that("list_ccsr_versions returns expected structure", {
  skip_on_cran()  # Requires internet access
  skip_if_offline()  # Skip if no internet
  
  versions <- list_ccsr_versions()
  expect_s3_class(versions, "tbl_df")
  expect_true("type" %in% names(versions))
  expect_true("version" %in% names(versions))
})

test_that("list_ccsr_versions filters by type", {
  skip_on_cran()  # Requires internet access
  skip_if_offline()  # Skip if no internet
  
  dx_versions <- list_ccsr_versions("diagnosis")
  expect_true(all(dx_versions$type == "diagnosis"))
  
  pr_versions <- list_ccsr_versions("procedure")
  expect_true(all(pr_versions$type == "procedure"))
})

#' Read HCUP Summary Trend Table from Disk
#'
#' Reads a previously downloaded HCUP Summary Trend Table Excel file from disk.
#' If no file path is provided, automatically finds and reads cached files from
#' `download_trend_tables()`, with an interactive menu to select from available
#' tables.
#'
#' @param file_path Optional character string, path to a trend table Excel file (.xlsx).
#'   If NULL (default), automatically searches the cache directory for files
#'   downloaded via `download_trend_tables()` and shows an interactive menu.
#' @param table_id Optional character string, table ID (e.g., "1", "2a", "2b") to
#'   read from cache. Only used when `file_path` is NULL. If both are NULL, shows
#'   interactive menu.
#' @param sheet Character string or integer specifying which sheet to read. If
#'   NULL (default), shows an interactive menu to select a sheet (in interactive sessions),
#'   or automatically selects the "National" sheet (or first data sheet) in non-interactive sessions.
#'   Common sheet names include "National", "Regional", "State", etc.
#' @param clean_names Logical. If TRUE (default), column names are cleaned to
#'   follow R naming conventions (snake_case).
#' @param as_data_table Logical or NULL. If TRUE and the `data.table` package is
#'   available, returns a `data.table` object instead of a tibble. If FALSE,
#'   returns a tibble. If NULL (default), prompts the user interactively to
#'   choose (only in interactive sessions). In non-interactive sessions, defaults
#'   to FALSE. Note: tibbles are already data frames and work with all standard
#'   R data frame operations.
#' @param name Optional character string, suggested variable name for the
#'   returned data. This is only used for display/messaging purposes and does
#'   not automatically assign the data to a variable. You must still assign the
#'   result: `my_data <- read_trend_table()`. If NULL (default), a name is
#'   suggested based on the table ID and sheet.
#'
#' @return A tibble (or data.table if `as_data_table = TRUE`) containing the
#'   trend table data. Tibbles are data frames and can be used with all
#'   standard R data frame operations, including `dplyr`, `data.table`, and
#'   base R functions.
#'
#' @note To use the data, assign it to a variable:
#'   `my_data <- read_trend_table()`. The `name` parameter is only for display
#'   purposes and does not automatically assign the data.
#'
#' @details
#' HCUP Summary Trend Tables are Excel files with multiple sheets containing
#' data at different geographic levels (National, Regional, State). Use the
#' `sheet` parameter to specify which sheet to read, or call the function
#' multiple times with different sheets.
#'
#' When `file_path` is NULL, the function automatically searches the cache
#' directory (`tempdir()`) for files matching the pattern `HCUP_SummaryTrendTables_*.xlsx`.
#' If multiple files are found, an interactive menu is displayed for selection.
#'
#' To see available sheets, use `list_trend_table_sheets()`.
#'
#' @examples
#' \donttest{
#' # Requires network: download a table, list sheets, read data (same file path)
#' path_xlsx <- download_trend_tables("2a")
#' list_trend_table_sheets(path_xlsx)
#' national_data <- read_trend_table(file_path = path_xlsx, as_data_table = FALSE)
#' head(national_data)
#'
#' # After a download, you can also read from cache by table ID
#' table_2a <- read_trend_table(table_id = "2a", as_data_table = FALSE)
#' head(table_2a)
#'
#' # With a file already on disk, pass its path to `read_trend_table(file_path = ...)`.
#' }
#'
#' @importFrom readxl read_excel excel_sheets
#' @importFrom tibble as_tibble
#' @export
read_trend_table <- function(file_path = NULL,
                            table_id = NULL,
                            sheet = NULL,
                            clean_names = TRUE,
                            as_data_table = NULL,
                            name = NULL) {
  
  # Determine suggested variable name early (before prompts)
  suggested_name <- determine_trend_table_name(name, table_id, sheet)
  
  # Handle interactive prompt for as_data_table FIRST (before file selection)
  # Show the tip along with the data.table question
  if (is.null(as_data_table)) {
    as_data_table <- prompt_for_data_table(suggested_name, "read_trend_table")
  }
  
  # Check if readxl is available
  if (!requireNamespace("readxl", quietly = TRUE)) {
    stop("Package 'readxl' is required for reading Excel files. ",
         "Install it with: install.packages('readxl')")
  }
  
  # If no file_path provided, automatically find cached file or show interactive menu
  if (is.null(file_path)) {
    cached_files <- list_cached_trend_table_files()
    
    if (length(cached_files) == 0) {
      stop("No cached trend table files found. ",
           "Please download a file first using `download_trend_tables()` ",
           "or provide a `file_path`.")
    }
    
    # If table_id is specified, try to find matching file
    if (!is.null(table_id)) {
      table_id <- tolower(as.character(table_id))
      matching_files <- cached_files[sapply(cached_files, function(f) {
        grepl(paste0("T", table_id, "\\.xlsx"), f$basename, ignore.case = TRUE)
      })]
      
      if (length(matching_files) > 0) {
        file_path <- matching_files[[1]]$path
        message("Reading cached file: ", basename(file_path))
      } else {
        stop("No cached file found for table_id '", table_id, 
             "'. Available files: ", 
             paste(sapply(cached_files, function(f) f$basename), collapse = ", "))
      }
    } else {
      # Multiple files - show interactive menu
      if (length(cached_files) == 1) {
        file_path <- cached_files[[1]]$path
        message("Reading cached file: ", basename(file_path))
      } else {
        file_path <- select_cached_trend_table_interactive(cached_files)
      }
    }
  }
  
  # Validate file path
  if (!file.exists(file_path)) {
    stop("File not found: ", file_path)
  }
  
  # Check if it's an Excel file
  if (!grepl("\\.xlsx?$", file_path, ignore.case = TRUE)) {
    stop("File must be an Excel file (.xlsx or .xls)")
  }
  
  # Get all sheet names
  all_sheets <- readxl::excel_sheets(file_path)
  
  # Identify metadata/disclaimer sheets to skip
  metadata_patterns <- c(
    "data.*use.*agreement",
    "disclaimer",
    "read.*me",
    "instructions",
    "notes",
    "metadata",
    "end.*of.*content"
  )
  
  is_metadata <- sapply(all_sheets, function(s) {
    any(sapply(metadata_patterns, function(p) {
      grepl(p, s, ignore.case = TRUE)
    }))
  })
  
  # Find data sheets (common names: National, Regional, State, etc.)
  data_sheets <- all_sheets[!is_metadata]
  
  # If sheet is specified, use it
  if (!is.null(sheet)) {
    if (is.character(sheet)) {
      if (!sheet %in% all_sheets) {
        stop("Sheet '", sheet, "' not found. Available sheets: ", 
             paste(all_sheets, collapse = ", "))
      }
      selected_sheet <- sheet
    } else if (is.numeric(sheet)) {
      if (sheet < 1 || sheet > length(all_sheets)) {
        stop("Sheet index ", sheet, " out of range. Available sheets: ", 
             length(all_sheets))
      }
      selected_sheet <- all_sheets[sheet]
    } else {
      stop("`sheet` must be a character string (sheet name) or integer (sheet index)")
    }
  } else {
    # No sheet specified - show interactive menu or auto-select
    if (interactive() && length(data_sheets) > 1) {
      # Show interactive menu for sheet selection
      selected_sheet <- select_trend_table_sheet_interactive(all_sheets, data_sheets, is_metadata)
    } else if (length(data_sheets) > 0) {
      # Non-interactive or single sheet: auto-select "National" or first data sheet
      if ("National" %in% data_sheets) {
        selected_sheet <- "National"
        message("Reading sheet: ", selected_sheet)
      } else {
        selected_sheet <- data_sheets[1]
        message("Reading sheet: ", selected_sheet)
      }
    } else {
      # Fallback: use first sheet if all are metadata
      selected_sheet <- all_sheets[1]
      warning("Could not identify data sheet. Using first sheet: ", selected_sheet)
    }
  }
  
  # Read the selected sheet
  data <- tryCatch({
    # Use "unique" name repair to handle empty column names
    # This will create unique names like "X1", "X2" for empty columns
    readxl::read_excel(file_path, sheet = selected_sheet, .name_repair = "unique")
  }, error = function(e) {
    stop("Failed to read Excel file: ", conditionMessage(e))
  })
  
  # Check if the sheet appears to be metadata (very few rows, mostly text)
  if (nrow(data) < 5) {
    # Check if it's mostly text/empty
    text_cols <- sapply(data, function(x) {
      if (is.character(x) || is.factor(x)) {
        sum(!is.na(x) & nchar(as.character(x)) > 50) / length(x)
      } else {
        0
      }
    })
    
    if (mean(text_cols) > 0.5) {
      warning("The selected sheet appears to be a metadata/disclaimer sheet. ",
              "Available data sheets: ", paste(data_sheets, collapse = ", "),
              ". Use `sheet` parameter to select a different sheet.")
    }
  }
  
  # Ensure it's a tibble with proper name repair (already done by read_excel, but ensure)
  if (!inherits(data, "tbl_df")) {
    data <- tibble::as_tibble(data, .name_repair = "unique")
  }
  
  # Clean column names if requested
  if (clean_names) {
    names(data) <- gsub("\\s+", "_", tolower(names(data)))
    names(data) <- gsub("[^a-z0-9_]", "", names(data))
  }
  
  # Convert to data.table if requested
  result <- convert_to_data_table_if_requested(data, as_data_table)
  
  # Show assignment message
  show_trend_table_assignment_message(suggested_name, result)
  
  return(result)
}

#' Determine suggested variable name for trend table data
#' @noRd
determine_trend_table_name <- function(name, table_id, sheet) {
  if (!is.null(name) && is.character(name) && length(name) == 1 && nchar(name) > 0) {
    # User provided a name, use it
    return(name)
  }
  
  # Generate suggested name based on table_id and sheet
  suggested <- "trend_table"
  
  if (!is.null(table_id)) {
    table_id_clean <- gsub("[^0-9a-z]", "", tolower(table_id))
    if (nchar(table_id_clean) > 0) {
      suggested <- paste0("table_", table_id_clean)
    }
  }
  
  if (!is.null(sheet)) {
    sheet_clean <- gsub("[^a-z0-9]", "", tolower(as.character(sheet)))
    if (nchar(sheet_clean) > 0) {
      suggested <- paste0(suggested, "_", sheet_clean)
    }
  }
  
  return(suggested)
}

#' Show message about how to assign the trend table data to a variable
#' @noRd
show_trend_table_assignment_message <- function(suggested_name, data) {
  if (interactive() && !is.null(suggested_name)) {
    data_type <- if (inherits(data, "data.table")) {
      "data.table"
    } else if (inherits(data, "tbl_df")) {
      "tibble"
    } else {
      "data.frame"
    }
    
    n_rows <- nrow(data)
    n_cols <- ncol(data)
    
    cat("\n")
    cat("=== Trend Table Imported Successfully ===\n")
    cat("Type: ", data_type, "\n", sep = "")
    cat("Dimensions: ", n_rows, " rows x ", n_cols, " columns\n", sep = "")
    cat("\n")
    cat("To use this data, assign it to a variable:\n")
    cat("  ", suggested_name, " <- read_trend_table(...)\n", sep = "")
    cat("\n")
    cat("Then you can use it:\n")
    cat("  head(", suggested_name, ")\n", sep = "")
    cat("  nrow(", suggested_name, ")\n", sep = "")
    cat("  # Or use with dplyr, data.table, etc.\n")
    cat("\n")
  }
}

#' List Available Sheets in Trend Table
#'
#' Lists all available sheets in a HCUP Summary Trend Table Excel file.
#'
#' @param file_path Character string, path to a trend table Excel file (.xlsx).
#'
#' @return A character vector of sheet names.
#'
#' @examples
#' \donttest{
#' # Requires network: download first, then list sheets from that file path
#' path_xlsx <- download_trend_tables("2a")
#' list_trend_table_sheets(path_xlsx)
#' }
#'
#' @importFrom readxl excel_sheets
#' @export
list_trend_table_sheets <- function(file_path) {
  if (!requireNamespace("readxl", quietly = TRUE)) {
    stop("Package 'readxl' is required. Install it with: install.packages('readxl')")
  }
  
  if (!file.exists(file_path)) {
    stop("File not found: ", file_path)
  }
  
  if (!grepl("\\.xlsx?$", file_path, ignore.case = TRUE)) {
    stop("File must be an Excel file (.xlsx or .xls)")
  }
  
  sheets <- readxl::excel_sheets(file_path)
  return(sheets)
}

#' List all cached trend table files
#' @noRd
list_cached_trend_table_files <- function() {
  cache_dir <- tempdir()
  
  # Look for trend table Excel files
  all_files <- list.files(cache_dir, 
                          pattern = "HCUP_SummaryTrendTables_.*\\.xlsx$", 
                          full.names = TRUE, 
                          ignore.case = TRUE)
  
  if (length(all_files) == 0) {
    return(list())
  }
  
  # Parse file information
  files_info <- list()
  for (file in all_files) {
    basename_file <- basename(file)
    
    # Extract table ID from filename (e.g., "T1", "T2a", "T2b")
    table_id_match <- regmatches(basename_file, 
                                regexpr("T([0-9]+[a-z]?)", basename_file, 
                                       ignore.case = TRUE))
    table_id <- if (length(table_id_match) > 0) {
      tolower(sub("T", "", table_id_match, ignore.case = TRUE))
    } else {
      "unknown"
    }
    
    files_info[[length(files_info) + 1]] <- list(
      path = file,
      table_id = table_id,
      basename = basename_file
    )
  }
  
  # Sort by table ID
  table_ids_numeric <- sapply(files_info, function(f) {
    id <- f$table_id
    # Convert "2a" to "2.1", "2b" to "2.2", etc. for sorting
    if (grepl("[a-z]$", id)) {
      num_part <- as.numeric(gsub("[a-z]", "", id))
      letter_part <- match(sub("[0-9]+", "", id), letters)
      paste0(num_part, ".", letter_part)
    } else {
      paste0(id, ".0")
    }
  })
  
  sorted_idx <- order(as.numeric(gsub("\\..*", "", table_ids_numeric)),
                     as.numeric(gsub(".*\\.", "", table_ids_numeric)))
  files_info <- files_info[sorted_idx]
  
  return(files_info)
}

#' Interactive selection of cached trend table file
#' @noRd
select_cached_trend_table_interactive <- function(cached_files) {
  # Show interactive menu
  cat("\n=== Available Cached Trend Table Files ===\n\n")
  for (i in seq_along(cached_files)) {
    f <- cached_files[[i]]
    cat(sprintf("%2d. Table %s - %s\n", 
                i, 
                toupper(f$table_id),
                f$basename))
  }
  cat("\n")
  
  # Get user selection
  if (interactive()) {
    selection <- readline("Select a file (enter number): ")
    selection <- as.integer(selection)
    
    if (is.na(selection) || selection < 1 || selection > length(cached_files)) {
      stop("Invalid selection. Please run the function again and select a valid number.")
    }
    
    return(cached_files[[selection]]$path)
  } else {
    # Non-interactive: return first file
    return(cached_files[[1]]$path)
  }
}

#' Prompt user for data.table preference (interactive only)
#' @noRd
prompt_for_data_table <- function(suggested_name = NULL, func_name = "read_ccsr") {
  # Check if data.table is available
  has_data_table <- requireNamespace("data.table", quietly = TRUE)
  
  if (!interactive()) {
    # Non-interactive: default to FALSE (tibble)
    return(FALSE)
  }
  
  if (!has_data_table) {
    # data.table not available: return FALSE
    return(FALSE)
  }
  
  # Interactive session with data.table available: ask user
  cat("\n")
  
  # Show tip about assigning to variable along with the data.table question
  if (!is.null(suggested_name)) {
    cat("Tip: Assign the result to a variable to use the data:\n")
    cat("  ", suggested_name, " <- ", func_name, "(...)\n", sep = "")
    cat("\n")
  }
  
  cat("Would you like to import as a data.table? (faster for large datasets)\n")
  cat("  [1] Yes (data.table)\n")
  cat("  [2] No (tibble/data.frame) - default\n")
  cat("Enter choice (1 or 2, or press Enter for default): ")
  
  choice <- readline()
  choice <- trimws(choice)
  
  if (choice == "1" || tolower(choice) == "y" || tolower(choice) == "yes") {
    return(TRUE)
  } else {
    return(FALSE)
  }
}

#' Convert tibble to data.table if requested
#' @noRd
convert_to_data_table_if_requested <- function(data, as_data_table) {
  if (as_data_table) {
    if (requireNamespace("data.table", quietly = TRUE)) {
      return(data.table::as.data.table(data))
    } else {
      warning("Package 'data.table' is not available. Returning tibble instead. ",
              "Install it with: install.packages('data.table')")
      return(data)
    }
  }
  return(data)
}

#' Interactive selection of trend table sheet
#' @noRd
select_trend_table_sheet_interactive <- function(all_sheets, data_sheets, is_metadata) {
  if (length(data_sheets) == 0) {
    warning("No data sheets found. All sheets appear to be metadata.")
    return(all_sheets[1])
  }
  
  if (length(data_sheets) == 1) {
    message("Reading sheet: ", data_sheets[1])
    return(data_sheets[1])
  }
  
  # Show interactive menu
  cat("\n=== Available Sheets ===\n\n")
  
  # Show data sheets first (recommended)
  cat("Data Sheets (recommended):\n")
  data_sheet_indices <- which(all_sheets %in% data_sheets)
  for (i in seq_along(data_sheets)) {
    idx <- data_sheet_indices[i]
    cat(sprintf("%2d. %s", idx, data_sheets[i]))
    if (data_sheets[i] == "National") {
      cat(" (default)")
    }
    cat("\n")
  }
  
  # Show metadata sheets if any (not recommended)
  if (any(is_metadata)) {
    metadata_sheets <- all_sheets[is_metadata]
    cat("\nMetadata/Disclaimer Sheets (not recommended):\n")
    metadata_indices <- which(is_metadata)
    for (i in seq_along(metadata_sheets)) {
      idx <- metadata_indices[i]
      cat(sprintf("%2d. %s (metadata)\n", idx, metadata_sheets[i]))
    }
  }
  
  cat("\n")
  cat("Select a sheet (enter number, or press Enter for 'National'): ")
  
  selection <- readline()
  selection <- trimws(selection)
  
  if (selection == "") {
    # Default to National if available, otherwise first data sheet
    if ("National" %in% data_sheets) {
      return("National")
    } else {
      return(data_sheets[1])
    }
  }
  
  selection <- as.integer(selection)
  
  if (is.na(selection) || selection < 1 || selection > length(all_sheets)) {
    stop("Invalid selection. Please run the function again and select a valid number.")
  }
  
  selected_sheet <- all_sheets[selection]
  
  # Warn if selecting a metadata sheet
  if (is_metadata[selection]) {
    warning("You selected a metadata/disclaimer sheet. ",
            "This may not contain the actual trend table data. ",
            "Consider selecting a data sheet instead.")
  }
  
  return(selected_sheet)
}

#' Map ICD-10 Codes to CCSR Categories
#'
#' Maps ICD-10-CM diagnosis codes or ICD-10-PCS procedure codes to their
#' corresponding CCSR categories using a downloaded CCSR mapping file.
#'
#' @param data A data frame or tibble containing ICD-10 codes to be mapped.
#' @param code_col Character string specifying the name of the column in `data`
#'   that contains the ICD-10 codes.
#' @param map_df A tibble containing the CCSR mapping data, typically obtained
#'   from `download_ccsr()`.
#' @param type Character string specifying the type of mapping. Must be one of:
#'   "diagnosis" (or "dx") for ICD-10-CM codes, or "procedure" (or "pr") for
#'   ICD-10-PCS codes. If NULL (default), the function will attempt to infer
#'   the type from the mapping data frame.
#' @param default_only Logical. For diagnosis codes only, if TRUE, returns only
#'   the default CCSR category (recommended for principal diagnosis analysis).
#'   If FALSE (default), returns all assigned CCSR categories including
#'   cross-classifications.
#' @param output_format Character string specifying the output format. Must be
#'   one of: "long" (default) or "wide". "long" format duplicates records for
#'   each assigned CCSR category. "wide" format creates multiple columns
#'   (CCSR_1, CCSR_2, etc.) for multiple categories.
#' @param keep_all Logical. If TRUE (default), returns all original columns
#'   from `data` plus the CCSR mapping columns. If FALSE, returns only the
#'   ICD-10 code column and CCSR mapping columns.
#'
#' @return A tibble with the original data plus CCSR mapping columns. The
#'   structure depends on `output_format`:
#'   - For "long" format: Each row represents one ICD-10 code and one CCSR
#'     category assignment (rows are duplicated for multiple categories).
#'   - For "wide" format: Each row represents one ICD-10 code with multiple
#'     CCSR category columns (CCSR_1, CCSR_2, etc.).
#'
#' @details
#' CCSR allows for cross-classification, meaning a single ICD-10 code can map
#' to multiple CCSR categories. The "long" format is recommended for analyses
#' where you want to count all assigned CCSR categories, while "wide" format
#' may be more convenient for patient-level analyses.
#'
#' For diagnosis codes, CCSR also assigns a "default" category that is
#' recommended for principal diagnosis analysis. Use `default_only = TRUE` to
#' extract only this default category.
#'
#' @examples
#' \donttest{
#' # Download mapping file
#' dx_map <- download_ccsr("diagnosis")
#'
#' # Create sample data
#' sample_data <- tibble::tibble(
#'   patient_id = 1:3,
#'   icd10_code = c("E11.9", "I10", "M79.3")
#' )
#'
#' # Map codes (long format - default)
#' mapped_long <- ccsr_map(
#'   data = sample_data,
#'   code_col = "icd10_code",
#'   map_df = dx_map
#' )
#'
#' # Map codes (wide format)
#' mapped_wide <- ccsr_map(
#'   data = sample_data,
#'   code_col = "icd10_code",
#'   map_df = dx_map,
#'   output_format = "wide"
#' )
#'
#' # Map codes (default category only)
#' mapped_default <- ccsr_map(
#'   data = sample_data,
#'   code_col = "icd10_code",
#'   map_df = dx_map,
#'   default_only = TRUE
#' )
#' }
#'
#' @importFrom dplyr left_join mutate group_by ungroup across all_of row_number desc
#' @importFrom tidyr pivot_wider pivot_longer
#' @importFrom tibble as_tibble
#' @importFrom stats setNames
#' @importFrom rlang .data
#' @export
ccsr_map <- function(data,
                    code_col,
                    map_df,
                    type = NULL,
                    default_only = FALSE,
                    output_format = "long",
                    keep_all = TRUE) {
  
  # Validate inputs
  if (!inherits(data, "data.frame")) {
    stop("`data` must be a data frame or tibble")
  }
  
  if (!code_col %in% names(data)) {
    stop("Column '", code_col, "' not found in `data`")
  }
  
  if (!inherits(map_df, "data.frame")) {
    stop("`map_df` must be a data frame or tibble")
  }
  
  if (!output_format %in% c("long", "wide")) {
    stop("`output_format` must be one of: 'long' or 'wide'")
  }
  
  # Infer type from mapping data if not provided
  if (is.null(type)) {
    type <- infer_ccsr_type(map_df)
  } else {
    type <- tolower(type)
    if (type %in% c("dx", "diagnosis")) {
      type <- "diagnosis"
    } else if (type %in% c("pr", "procedure")) {
      type <- "procedure"
    } else {
      stop("`type` must be one of: 'diagnosis'/'dx' or 'procedure'/'pr'")
    }
  }
  
  # Find relevant columns in mapping data
  map_cols <- identify_mapping_columns(map_df, type, default_only)
  
  if (is.null(map_cols$icd_col)) {
    stop("Could not identify ICD-10 code column in mapping data")
  }
  
  if (type == "diagnosis" && length(map_cols$ccsr_cols) == 0 && !default_only) {
    stop("Could not identify CCSR category columns in mapping data")
  }

  if (type == "procedure" && is.null(map_cols$ccsr_col)) {
    stop("Could not identify CCSR category column in mapping data")
  }
  
  # Prepare data for joining
  data_prep <- tibble::as_tibble(data)
  data_prep[[code_col]] <- format_icd_codes(data_prep[[code_col]])
  data_prep$.hcuptools_row_id <- seq_len(nrow(data_prep))
  
  mapping <- tibble::as_tibble(map_df)
  mapping <- normalize_ccsr_character_columns(mapping)
  mapping[[map_cols$icd_col]] <- format_icd_codes(mapping[[map_cols$icd_col]])
  
  if (type == "diagnosis") {
    mapping <- prepare_diagnosis_mapping(mapping, map_cols, default_only)
    join_cols <- c("icd_code", "ccsr_category", "ccsr_num", "is_default")
    join_by <- stats::setNames("icd_code", code_col)
  } else {
    join_cols <- c(map_cols$icd_col, map_cols$ccsr_col)
    if (!is.null(map_cols$desc_col)) {
      join_cols <- c(join_cols, map_cols$desc_col)
    }
    join_by <- stats::setNames(map_cols$icd_col, code_col)
    mapping <- mapping[, join_cols, drop = FALSE]
  }
  
  # Perform the join
  result <- dplyr::left_join(
    data_prep,
    mapping,
    by = join_by
  )
  
  # Handle output format
  if (output_format == "wide" && type == "diagnosis") {
    result <- reshape_to_wide(result, code_col)
  } else if (type == "diagnosis") {
    result$ccsr_num <- NULL
  }
  
  # Select columns if keep_all is FALSE
  if (!keep_all) {
    if (type == "diagnosis") {
      ccsr_keep_cols <- grep("^CCSR_\\d+$|^ccsr_category$|^is_default$", names(result), value = TRUE)
      keep_cols <- c(code_col, ccsr_keep_cols)
    } else {
      keep_cols <- c(code_col, map_cols$ccsr_col)
      if (!is.null(map_cols$desc_col)) {
        keep_cols <- c(keep_cols, map_cols$desc_col)
      }
    }
    result <- result[, names(result) %in% keep_cols, drop = FALSE]
  }

  result$.hcuptools_row_id <- NULL
  
  return(result)
}

#' Infer CCSR type from mapping data frame
#'
#' @noRd
infer_ccsr_type <- function(map_df) {
  col_names <- tolower(names(map_df))
  
  # Check for diagnosis indicators
  dx_indicators <- c("dxccsr", "diagnosis", "icd.*10.*cm", "dx.*ccsr")
  if (any(sapply(dx_indicators, function(p) any(grepl(p, col_names))))) {
    return("diagnosis")
  }
  
  # Check for procedure indicators
  pr_indicators <- c("prccsr", "procedure", "icd.*10.*pcs", "pr.*ccsr")
  if (any(sapply(pr_indicators, function(p) any(grepl(p, col_names))))) {
    return("procedure")
  }
  
  # Check column values - look for common diagnosis vs procedure patterns
  # Diagnosis codes typically have dots (E11.9), procedures don't
  code_col <- grep("icd|code|dx|pr", col_names, value = TRUE)[1]
  if (!is.na(code_col) && code_col %in% names(map_df)) {
    sample_codes <- utils::head(map_df[[code_col]][!is.na(map_df[[code_col]])], 100)
    if (length(sample_codes) > 0) {
      # Check if codes contain dots (diagnosis) or are alphanumeric without dots (procedure)
      has_dots <- any(grepl("\\.", sample_codes))
      if (has_dots) {
        return("diagnosis")
      } else if (any(grepl("^[0-9]", sample_codes))) {
        # Procedure codes often start with numbers
        return("procedure")
      }
    }
  }
  
  # Default to diagnosis if unclear (but suppress warning if we have some evidence)
  # Only warn if we really can't tell
  if (is.null(code_col)) {
    warning("Could not infer CCSR type from mapping data, defaulting to 'diagnosis'")
  }
  return("diagnosis")
}

#' Identify relevant columns in mapping data frame
#'
#' @noRd
identify_mapping_columns <- function(map_df, type, default_only) {
  col_names <- tolower(names(map_df))
  
  result <- list(icd_col = NULL, ccsr_col = NULL, ccsr_cols = character(0), default_col = NULL, desc_col = NULL)
  
  # Find ICD code column
  if (type == "diagnosis") {
    icd_patterns <- c("icd.*10.*cm", "icd.*10", "diagnosis.*code", "^code$", "^dx$")
  } else {
    icd_patterns <- c("icd.*10.*pcs", "icd.*10", "procedure.*code", "^code$", "^pr$")
  }
  
  for (pattern in icd_patterns) {
    match <- grep(pattern, col_names, value = TRUE)
    if (length(match) > 0) {
      result$icd_col <- match[1]
      break
    }
  }
  
  # Find CCSR category column(s)
  if (type == "procedure") {
    ccsr_patterns <- c("^prccsr$", "prccsr", "ccsr.*category", "ccsr.*code", "^ccsr$", "category")
    for (pattern in ccsr_patterns) {
      match <- grep(pattern, col_names, value = TRUE)
      if (length(match) > 0) {
        result$ccsr_col <- match[1]
        break
      }
    }
  } else {
    diagnosis_slot_cols <- grep("^ccsr.*category.*[1-6]$", col_names, value = TRUE)
    if (length(diagnosis_slot_cols) > 0) {
      slot_nums <- suppressWarnings(as.integer(gsub(".*?([1-6])$", "\\1", diagnosis_slot_cols)))
      diagnosis_slot_cols <- diagnosis_slot_cols[order(slot_nums)]
      result$ccsr_cols <- diagnosis_slot_cols
      result$ccsr_col <- diagnosis_slot_cols[1]
    }
  }
  
  # Find default CCSR column (diagnosis only)
  if (type == "diagnosis") {
    default_patterns <- c("default.*ccsr", "default.*category", "default")
    for (pattern in default_patterns) {
      match <- grep(pattern, col_names, value = TRUE)
      if (length(match) > 0) {
        result$default_col <- match[1]
        break
      }
    }
  }
  
  # Find description column
  desc_patterns <- c("description", "label", "name", "desc")
  for (pattern in desc_patterns) {
    match <- grep(pattern, col_names, value = TRUE)
    if (length(match) > 0) {
      result$desc_col <- match[1]
      break
    }
  }
  
  return(result)
}

#' Reshape result to wide format
#'
#' @noRd
reshape_to_wide <- function(data, code_col) {
  id_cols <- setdiff(names(data), c("ccsr_category", "ccsr_num", "is_default"))
  base_rows <- unique(data[, id_cols, drop = FALSE])
  matched_rows <- data[!is.na(data$ccsr_num), , drop = FALSE]

  if (nrow(matched_rows) == 0) {
    return(base_rows)
  }

  wide_map <- matched_rows |>
    tidyr::pivot_wider(
      id_cols = dplyr::all_of(id_cols),
      names_from = ccsr_num,
      values_from = dplyr::all_of("ccsr_category"),
      names_prefix = "CCSR_"
    )
  result <- dplyr::left_join(base_rows, wide_map, by = id_cols)
  
  return(result)
}

#' Prepare diagnosis mapping for ICD-to-CCSR joins
#'
#' @noRd
prepare_diagnosis_mapping <- function(mapping, map_cols, default_only) {
  if (length(map_cols$ccsr_cols) == 0 && is.null(map_cols$default_col)) {
    stop("Could not identify diagnosis CCSR columns in mapping data")
  }

  if (length(map_cols$ccsr_cols) > 0) {
    long_map <- mapping |>
      tidyr::pivot_longer(
        cols = dplyr::all_of(map_cols$ccsr_cols),
        names_to = "ccsr_source_col",
        values_to = "ccsr_category"
      )
    long_map$ccsr_num <- suppressWarnings(as.integer(gsub(".*?([1-6])$", "\\1", long_map$ccsr_source_col)))
  } else {
    long_map <- mapping
    long_map$ccsr_category <- NA_character_
    long_map$ccsr_num <- NA_integer_
  }

  long_map$ccsr_category <- strip_surrounding_quotes(long_map$ccsr_category)
  long_map$icd_code <- format_icd_codes(long_map[[map_cols$icd_col]])

  if (!is.null(map_cols$default_col)) {
    default_vals <- strip_surrounding_quotes(long_map[[map_cols$default_col]])
    long_map$is_default <- !is.na(long_map$ccsr_category) &
      nzchar(long_map$ccsr_category) &
      long_map$ccsr_category == default_vals
  } else {
    long_map$is_default <- NA
  }

  long_map <- long_map[!is.na(long_map$ccsr_category) & nzchar(long_map$ccsr_category), , drop = FALSE]

  if (default_only) {
    if (!is.null(map_cols$default_col)) {
      long_map <- long_map[!is.na(long_map$is_default) & long_map$is_default, , drop = FALSE]
    } else {
      long_map <- long_map[0, , drop = FALSE]
    }
  }

  unique(long_map[, c("icd_code", "ccsr_category", "ccsr_num", "is_default"), drop = FALSE])
}

# Declare global variables to avoid R CMD check notes
utils::globalVariables("ccsr_num")

# Test file for ccsr_map function

test_that("ccsr_map validates inputs", {
  # Create sample data
  sample_data <- tibble::tibble(
    id = 1:3,
    code = c("E11.9", "I10", "M79.3")
  )
  
  # Create sample mapping data
  sample_map <- tibble::tibble(
    icd_code = c("E11.9", "I10", "M79.3"),
    ccsr_category = c("END001", "CIR019", "MUS003")
  )
  
  # Test invalid data
  expect_error(ccsr_map(data = "not a dataframe", code_col = "code", map_df = sample_map),
               "must be a data frame")
  
  # Test invalid code column
  expect_error(ccsr_map(data = sample_data, code_col = "nonexistent", map_df = sample_map),
               "not found in")
  
  # Test invalid output format
  expect_error(ccsr_map(data = sample_data, code_col = "code", map_df = sample_map,
                        output_format = "invalid"),
               "must be one of")
})

test_that("ccsr_map handles output formats", {
  sample_data <- tibble::tibble(
    id = 1:2,
    code = c("A000", "B111")
  )

  sample_map <- tibble::tibble(
    icd_10_cm_code = c("'A000'", "B111"),
    ccsr_category_1 = c("'DIG001'", "RSP002"),
    ccsr_category_2 = c("'INF003'", NA_character_),
    ccsr_category_3 = c(NA_character_, NA_character_),
    ccsr_category_4 = c(NA_character_, NA_character_),
    ccsr_category_5 = c(NA_character_, NA_character_),
    ccsr_category_6 = c(NA_character_, NA_character_),
    default_ccsr_category = c("'DIG001'", "RSP002")
  )

  mapped_long <- ccsr_map(
    data = sample_data,
    code_col = "code",
    map_df = sample_map,
    output_format = "long",
    default_only = FALSE
  )

  a000_rows <- mapped_long[mapped_long$code == "A000", ]
  expect_equal(sort(a000_rows$ccsr_category), c("DIG001", "INF003"))
  expect_true(any(a000_rows$is_default))
  expect_equal(sum(a000_rows$is_default), 1)

  mapped_default <- ccsr_map(
    data = sample_data,
    code_col = "code",
    map_df = sample_map,
    output_format = "long",
    default_only = TRUE
  )

  expect_equal(mapped_default$ccsr_category[mapped_default$code == "A000"], "DIG001")
  expect_true(all(mapped_default$is_default %in% c(TRUE, NA)))

  mapped_wide <- ccsr_map(
    data = sample_data,
    code_col = "code",
    map_df = sample_map,
    output_format = "wide",
    default_only = FALSE
  )

  expect_true(all(c("CCSR_1", "CCSR_2") %in% names(mapped_wide)))
  expect_equal(mapped_wide$CCSR_1[mapped_wide$code == "A000"], "DIG001")
  expect_equal(mapped_wide$CCSR_2[mapped_wide$code == "A000"], "INF003")
})

#' Get CCSR Category Descriptions
#'
#' Retrieves the full clinical description for one or more CCSR category codes.
#' This function helps users interpret CCSR codes by providing their meaningful
#' clinical descriptions.
#'
#' @param ccsr_codes Character vector of CCSR category codes (e.g., "ADM010",
#'   "NEP003", "CIR019").
#' @param map_df Optional. A tibble containing CCSR mapping data with
#'   descriptions. If provided, descriptions are extracted from this data frame.
#'   If NULL (default), the function will attempt to download the latest mapping
#'   file to extract descriptions.
#' @param type Character string specifying the type of CCSR codes. Must be one
#'   of: "diagnosis" (or "dx") or "procedure" (or "pr"). If NULL (default), the
#'   function will attempt to infer the type from the codes or mapping data.
#'
#' @return A tibble with columns:
#'   - `ccsr_code`: The CCSR category code
#'   - `description`: The full clinical description
#'   - Additional metadata columns if available in the mapping data
#'
#' @details
#' CCSR category codes follow specific naming conventions:
#' - Diagnosis codes: Typically start with letters (e.g., "ADM010", "NEP003")
#' - Procedure codes: Typically start with letters (e.g., "PRC001", "PRC002")
#'
#' If a description is not found for a code, it will be marked as NA in the
#' result.
#'
#' @examples
#' \donttest{
#' # Get descriptions using downloaded mapping data
#' dx_map <- download_ccsr("diagnosis")
#' get_ccsr_description(c("ADM010", "NEP003", "CIR019"), map_df = dx_map)
#'
#' # Get descriptions without pre-downloaded data (will download automatically)
#' get_ccsr_description(c("ADM010", "NEP003"), type = "diagnosis")
#' }
#'
#' @importFrom dplyr distinct filter left_join
#' @importFrom tibble tibble
#' @importFrom stats setNames
#' @importFrom rlang .data
#' @export
get_ccsr_description <- function(ccsr_codes,
                                 map_df = NULL,
                                 type = NULL) {
  
  # Validate inputs
  if (!is.character(ccsr_codes) || length(ccsr_codes) == 0) {
    stop("`ccsr_codes` must be a non-empty character vector")
  }
  ccsr_codes <- strip_surrounding_quotes(ccsr_codes)
  
  # If map_df is not provided, download it
  if (is.null(map_df)) {
    if (is.null(type)) {
      # Try to infer type from codes (simple heuristic)
      if (any(grepl("^PRC", ccsr_codes, ignore.case = TRUE))) {
        type <- "procedure"
      } else {
        type <- "diagnosis"  # Default
      }
    }
    
    message("Downloading CCSR mapping file to extract descriptions...")
    map_df <- download_ccsr(type = type, version = "latest")
  }
  
  # Infer type from map_df if not provided
  if (is.null(type)) {
    type <- infer_ccsr_type(map_df)
  }
  
  # Find description column
  col_names <- tolower(names(map_df))
  desc_col <- NULL
  
  desc_patterns <- c("ccsr.*description", "description", "label", "name", "desc")
  for (pattern in desc_patterns) {
    match <- grep(pattern, col_names, value = TRUE)
    if (length(match) > 0) {
      desc_col <- match[1]
      break
    }
  }
  
  if (is.null(desc_col)) {
    stop("Could not find description column in mapping data")
  }
  
  # Find CCSR code column
  ccsr_col <- NULL
  ccsr_patterns <- c("ccsr.*category", "ccsr.*code", "^ccsr$", "category")
  for (pattern in ccsr_patterns) {
    match <- grep(pattern, col_names, value = TRUE)
    if (length(match) > 0) {
      ccsr_col <- match[1]
      break
    }
  }
  
  if (is.null(ccsr_col)) {
    stop("Could not find CCSR code column in mapping data")
  }
  
  # Extract unique descriptions
  unique_map <- map_df[, c(ccsr_col, desc_col)] |>
    dplyr::distinct() |>
    dplyr::filter(!is.na(.data[[ccsr_col]]))
  unique_map[[ccsr_col]] <- strip_surrounding_quotes(unique_map[[ccsr_col]])
  
  # Match codes to descriptions
  join_by <- stats::setNames(ccsr_col, "ccsr_code")
  result <- tibble::tibble(ccsr_code = ccsr_codes) |>
    dplyr::left_join(
      unique_map,
      by = join_by
    )
  
  # Rename description column for clarity
  names(result)[names(result) == desc_col] <- "description"
  
  # Check for unmatched codes
  unmatched <- is.na(result$description)
  if (any(unmatched)) {
    warning("No description found for ", sum(unmatched), " code(s): ",
            paste(ccsr_codes[unmatched], collapse = ", "))
  }
  
  return(result)
}

#' Generate Citation for HCUP Resources
#'
#' Provides recommended citations for HCUP resources including Clinical
#' Classifications Software Refined (CCSR) data and Summary Trend Tables from
#' the Agency for Healthcare Research and Quality (AHRQ) Healthcare Cost and
#' Utilization Project (HCUP).
#'
#' @param format Character string specifying the citation format. Must be one
#'   of: "text" (default), "bibtex", or "r" (for R citation object).
#' @param version Character string specifying the CCSR version to cite. If
#'   "latest" (default), the function will attempt to fetch the latest version
#'   from the HCUP website. Otherwise, specify a version like "v2026.1".
#' @param resource Character string specifying which HCUP resource to cite.
#'   Options: "ccsr" (default) for CCSR data, or "trend_tables" for Summary
#'   Trend Tables.
#'
#' @return If `format` is "text", returns a character string with the citation.
#'   If `format` is "bibtex", returns a character string with BibTeX format.
#'   If `format` is "r", returns an R citation object.
#'
#' @details
#' This function generates citations for HCUP resources following AHRQ/HCUP
#' guidelines. The citation includes the appropriate version number and
#' access date. For CCSR data, the version is automatically detected if not
#' specified. For Summary Trend Tables, the citation references the general
#' HCUP Summary Trend Tables resource.
#'
#' @examples
#' # Text citation for CCSR
#' hcup_citation()
#'
#' # BibTeX format for CCSR
#' hcup_citation(format = "bibtex")
#'
#' # Citation for Summary Trend Tables
#' hcup_citation(resource = "trend_tables")
#'
#' # R citation object
#' hcup_citation(format = "r")
#'
#' @importFrom utils person
#' @export
hcup_citation <- function(format = "text", version = "latest", resource = "ccsr") {
  # Validate format
  if (!format %in% c("text", "bibtex", "r")) {
    stop("`format` must be one of: 'text', 'bibtex', or 'r'")
  }
  
  # Validate resource
  resource <- tolower(resource)
  if (!resource %in% c("ccsr", "trend_tables", "trend")) {
    stop("`resource` must be one of: 'ccsr' or 'trend_tables'")
  }
  
  # Get current date for access date
  access_date <- format(Sys.Date(), "%B %d, %Y")
  
  # Handle Summary Trend Tables citation
  if (resource %in% c("trend_tables", "trend")) {
    if (format == "text") {
      citation_text <- paste0(
        "Agency for Healthcare Research and Quality. HCUP Summary Trend Tables. ",
        "Healthcare Cost and Utilization Project (HCUP). ",
        "Agency for Healthcare Research and Quality, Rockville, MD. ",
        "www.hcup-us.ahrq.gov/reports/trendtables/summarytrendtables.jsp. ",
        "Accessed ", access_date, ". "
      )
      return(citation_text)
    } else if (format == "bibtex") {
      citation_bibtex <- paste0(
        "@misc{hcup_trend_tables,\n",
        "  title = {HCUP Summary Trend Tables},\n",
        "  author = {{Agency for Healthcare Research and Quality}},\n",
        "  organization = {Healthcare Cost and Utilization Project (HCUP)},\n",
        "  publisher = {Agency for Healthcare Research and Quality},\n",
        "  address = {Rockville, MD},\n",
        "  year = {", format(Sys.Date(), "%Y"), "},\n",
        "  url = {https://hcup-us.ahrq.gov/reports/trendtables/summarytrendtables.jsp},\n",
        "  note = {Accessed ", access_date, "}\n",
        "}"
      )
      return(citation_bibtex)
    } else if (format == "r") {
      # Return R citation object for Trend Tables
      citation_r <- utils::bibentry(
        "Misc",
        title = "HCUP Summary Trend Tables",
        author = utils::person("Agency for Healthcare Research and Quality", role = "aut"),
        year = format(Sys.Date(), "%Y"),
        note = paste0("Accessed ", access_date),
        url = "https://hcup-us.ahrq.gov/reports/trendtables/summarytrendtables.jsp"
      )
      return(citation_r)
    }
  }
  
  # Handle CCSR citation (original functionality)
  # Get version if needed
  if (version == "latest") {
    tryCatch({
      version <- get_latest_version("diagnosis")
    }, error = function(e) {
      version <- "v2026.1"  # Fallback version
    })
  }
  
  # Validate version format
  if (!grepl("^v\\d{4}[-.]\\d+$", version)) {
    stop("`version` must be in format 'vYYYY.N' or 'vYYYY-N' (e.g., 'v2026.1')")
  }
  
  # Normalize version for display (use dot format)
  version_display <- gsub("-", ".", version)
  
  if (format == "text") {
    citation_text <- paste0(
      "Agency for Healthcare Research and Quality. Clinical Classifications Software Refined (CCSR) for ICD-10-CM Diagnoses, ",
      version_display, ". Healthcare Cost and Utilization Project (HCUP). ",
      "Agency for Healthcare Research and Quality, Rockville, MD. ",
      "www.hcup-us.ahrq.gov/toolssoftware/ccsr/ccs_refined.jsp. Accessed ", access_date, ". "
    )
    return(citation_text)
  } else if (format == "bibtex") {
    citation_bibtex <- paste0(
      "@misc{hcup_ccsr_", gsub("[^0-9]", "", version_display), ",\n",
      "  title = {Clinical Classifications Software Refined (CCSR) for ICD-10-CM Diagnoses, ", version_display, "},\n",
      "  author = {{Agency for Healthcare Research and Quality}},\n",
      "  organization = {Healthcare Cost and Utilization Project (HCUP)},\n",
      "  publisher = {Agency for Healthcare Research and Quality},\n",
      "  address = {Rockville, MD},\n",
      "  year = {", sub("v(\\d{4})\\.(\\d+)", "\\1", version_display), "},\n",
      "  url = {https://www.hcup-us.ahrq.gov/toolssoftware/ccsr/ccs_refined.jsp},\n",
      "  note = {Accessed ", access_date, "}\n",
      "}"
    )
    return(citation_bibtex)
  } else if (format == "r") {
    # Return R citation object for CCSR
    citation_r <- utils::bibentry(
      "Misc",
      title = paste0("Clinical Classifications Software Refined (CCSR) for ICD-10-CM Diagnoses, ", version_display),
      author = utils::person("Agency for Healthcare Research and Quality", role = "aut"),
      year = sub("v(\\d{4})\\.(\\d+)", "\\1", version_display),
      note = paste0("Accessed ", access_date),
      url = "https://www.hcup-us.ahrq.gov/toolssoftware/ccsr/ccs_refined.jsp"
    )
    return(citation_r)
  }
}

#' Format ICD codes (preserve leading zeros)
#'
#' @param codes Character or numeric vector of ICD codes
#' @return Character vector of formatted ICD codes
#'
#' @noRd
format_icd_codes <- function(codes) {
  codes <- strip_surrounding_quotes(codes)
  return(trimws(as.character(codes)))
}

#' List Available CCSR Versions
#'
#' Returns a list of available CCSR versions for download by scraping the HCUP
#' website. This function helps users identify which versions are available for
#' diagnosis and procedure mapping files.
#'
#' @param type Character string specifying the type of CCSR file. Must be one
#'   of: "diagnosis" (or "dx"), "procedure" (or "pr"), or "all" (default) to
#'   list versions for both types.
#'
#' @return A data frame (tibble) with columns:
#'   - `type`: The CCSR type ("diagnosis" or "procedure")
#'   - `version`: The version identifier (e.g., "v2026.1")
#'
#' @details
#' This function fetches available CCSR versions from the HCUP website.
#' Results are cached for 24 hours to minimize website requests. If the website
#' cannot be accessed, the function will return an error.
#'
#' @examples
#' \donttest{
#' # List all available versions
#' list_ccsr_versions()
#'
#' # List only diagnosis versions
#' list_ccsr_versions("diagnosis")
#'
#' # List only procedure versions
#' list_ccsr_versions("procedure")
#' }
#'
#' @importFrom dplyr bind_rows mutate arrange select desc
#' @importFrom rlang .data
#' @importFrom tibble tibble
#' @export
list_ccsr_versions <- function(type = "all") {
  # Validate type
  type <- tolower(type)
  if (type %in% c("dx", "diagnosis")) {
    type <- "diagnosis"
  } else if (type %in% c("pr", "procedure")) {
    type <- "procedure"
  } else if (type != "all") {
    stop("`type` must be one of: 'all', 'diagnosis'/'dx', or 'procedure'/'pr'")
  }
  
  # Cache key for storing results
  cache_name <- "ccsr_all_versions"
  cache_path <- file.path(tempdir(), cache_name)
  
  # Check cache (valid for 24 hours)
  all_versions <- NULL
  if (file.exists(cache_path)) {
    cache_info <- file.info(cache_path)
    cache_age <- as.numeric(Sys.time() - cache_info$mtime, units = "hours")
    if (cache_age < 24) {
      tryCatch({
        cached_data <- readRDS(cache_path)
        if (inherits(cached_data, "data.frame") && 
            "type" %in% names(cached_data) && 
            "version" %in% names(cached_data)) {
          all_versions <- cached_data
        }
      }, error = function(e) {
        # Cache file corrupted, will re-fetch
      })
    }
  }
  
  # If not cached, fetch from HCUP website
  if (is.null(all_versions)) {
    dx_versions <- unique(c(
      try_fetch_all_versions_from_url("DXCCSR"),
      try_fetch_versions_direct("DXCCSR")
    ))
    pr_versions <- unique(c(
      try_fetch_all_versions_from_url("PRCCSR"),
      try_fetch_versions_direct("PRCCSR")
    ))
    
    # Combine results
    result_list <- list()
    if (!is.null(dx_versions) && length(dx_versions) > 0) {
      result_list <- c(result_list, list(
        tibble::tibble(type = "diagnosis", version = dx_versions)
      ))
    }
    if (!is.null(pr_versions) && length(pr_versions) > 0) {
      result_list <- c(result_list, list(
        tibble::tibble(type = "procedure", version = pr_versions)
      ))
    }
    
    if (length(result_list) > 0) {
      all_versions <- dplyr::bind_rows(result_list)
      # Sort by version (newest first)
      all_versions <- all_versions |>
        dplyr::mutate(
          version_num = as.numeric(gsub("v(\\d{4})[-.](\\d+)", "\\1.\\2", .data$version))
        ) |>
        dplyr::arrange(dplyr::desc(.data$version_num)) |>
        dplyr::select(-.data$version_num)
      
      # Cache the result
      tryCatch({
        saveRDS(all_versions, cache_path)
      }, error = function(e) {
        # Cache save failed, continue without caching
      })
    } else {
      stop("Could not retrieve CCSR versions from HCUP website. ",
           "Please check your internet connection and try again.")
    }
  }
  
  # Filter by type if specified
  if (type == "diagnosis") {
    return(all_versions[all_versions$type == "diagnosis", ])
  } else if (type == "procedure") {
    return(all_versions[all_versions$type == "procedure", ])
  } else {
    return(all_versions)
  }
}

#' Fetch all versions from HCUP website for a specific file prefix
#'
#' @param file_prefix Character string, either "DXCCSR" or "PRCCSR"
#' @return Character vector of version strings, or NULL if error
#'
#' @noRd
try_fetch_all_versions_from_url <- function(file_prefix) {
  # Try main page first
  versions <- try_fetch_versions_from_url(
    "https://hcup-us.ahrq.gov/toolssoftware/ccsr/", 
    file_prefix
  )
  if (!is.null(versions) && length(versions) > 0) {
    return(versions)
  }
  
  # Try archive page
  versions <- try_fetch_versions_from_url(
    "https://hcup-us.ahrq.gov/toolssoftware/ccsr/ccsr_archive.jsp", 
    file_prefix
  )
  if (!is.null(versions) && length(versions) > 0) {
    return(versions)
  }
  
  return(NULL)
}

#' Get CCSR Change Log
#'
#' Retrieves and displays the change log for CCSR versions. The change log
#' documents updates, additions, and modifications to CCSR categories across
#' different versions.
#'
#' @param version Character string specifying the CCSR version. Use "latest"
#'   (default) to get the change log for the most recent version, or specify a
#'   version like "v2026.1", "v2025.1", etc.
#' @param type Character string specifying the type of CCSR. Must be one of:
#'   "diagnosis" (or "dx") for ICD-10-CM diagnosis codes, or "procedure" (or
#'   "pr") for ICD-10-PCS procedure codes. Default is "diagnosis".
#' @param format Character string specifying the output format. Options:
#'   "read" (default) - Downloads and reads the Excel file as a data table/tibble (requires `readxl` package)
#'   "text" - Returns change log information as text
#'   "url" - Returns the URL to the change log document
#'   "download" - Downloads and returns the change log file path
#'   "view" - Downloads and opens the change log file in the default viewer
#'   "extract" - Attempts to extract text from the file (requires `pdftools` for PDF or `readxl` for Excel)
#'
#' @return Depending on `format`:
#'   - "read" (default): A tibble or data.table containing the change log data (if Excel file)
#'   - "text": Character string with change log information
#'   - "url": Character string with URL to change log
#'   - "download": Character string with path to downloaded file
#'   - "view": Opens the file and returns the file path (invisibly)
#'   - "extract": Character string with extracted text from file
#'
#' @param as_data_table Logical. If TRUE, returns a `data.table` instead of a tibble.
#'   Only used when `format = "read"`. If NULL (default), prompts the user interactively
#'   to choose (only in interactive sessions). In non-interactive sessions, defaults to FALSE.
#'
#' @details
#' CCSR change logs document:
#' - New CCSR categories added
#' - Categories that were removed or merged
#' - Changes to category descriptions
#' - Updates to ICD-10 code mappings
#' - Version-specific notes and improvements
#'
#' Change logs are typically available as PDF or text documents on the HCUP
#' website. This function attempts to locate and retrieve them.
#'
#' @examples
#' \donttest{
#' # Get latest change log URL
#' changelog_url <- ccsr_changelog(format = "url")
#'
#' # Get change log information
#' changelog_info <- ccsr_changelog(version = "v2026.1", format = "text")
#'
#' # Download change log file
#' changelog_file <- ccsr_changelog(version = "v2025.1", format = "download")
#'
#' # View change log in default PDF viewer
#' ccsr_changelog(version = "v2026.1", format = "view")
#'
#' # Extract text from change log PDF (requires pdftools package)
#' changelog_text <- ccsr_changelog(version = "v2026.1", format = "extract")
#' cat(changelog_text)
#' }
#'
#' @importFrom httr2 request req_timeout req_user_agent req_perform resp_body_string resp_body_raw
#' @importFrom xml2 read_html xml_find_all xml_attr xml_text
#' @importFrom readxl read_excel excel_sheets
#' @export
ccsr_changelog <- function(version = "latest",
                          type = "diagnosis",
                          format = "read",
                          as_data_table = NULL) {
  requested_version <- version
  
  # Validate inputs
  type <- tolower(type)
  if (type %in% c("dx", "diagnosis")) {
    type <- "diagnosis"
  } else if (type %in% c("pr", "procedure")) {
    type <- "procedure"
  } else {
    stop("`type` must be one of: 'diagnosis'/'dx' or 'procedure'/'pr'")
  }
  
  if (!format %in% c("text", "url", "download", "view", "extract", "read")) {
    stop("`format` must be one of: 'text', 'url', 'download', 'view', 'extract', or 'read'")
  }
  
  # Get version (use internal function from download_ccsr.R)
  if (version == "latest") {
    # Use the internal get_latest_version function
    version <- get_latest_version(type)
  }
  
  # Validate version format
  if (!grepl("^v\\d{4}[-.]\\d+$", version)) {
    stop("`version` must be in format 'vYYYY.N' or 'vYYYY-N' (e.g., 'v2026.1')")
  }
  
  # Convert version format for matching
  # Version format: v2026.1 -> v20261 (for filename matching)
  version_compact <- gsub("[^0-9]", "", version)
  version_compact <- paste0("v", version_compact)
  
  # Base URL for CCSR resources
  base_url <- "https://hcup-us.ahrq.gov/toolssoftware/ccsr/"
  
  # Dynamically discover change log files from HCUP website
  changelog_urls <- character()
  
  # Try to fetch from CCSR main page and archive page
  urls_to_check <- c(
    "https://hcup-us.ahrq.gov/toolssoftware/ccsr/ccsr_refined.jsp",
    "https://hcup-us.ahrq.gov/toolssoftware/ccsr/ccsr_archive.jsp"
  )
  
  # Determine prefix for file matching
  if (type == "diagnosis") {
    prefix <- "DXCCSR"
  } else {
    prefix <- "PRCCSR"
  }
  
  # Try each URL to find change log links
  for (check_url in urls_to_check) {
    if (length(changelog_urls) > 0) break
    
    tryCatch({
      resp <- httr2::request(check_url) |>
        httr2::req_user_agent("HCUPtools R package") |>
        httr2::req_timeout(30) |>
        httr2::req_perform()
      
      html_content <- httr2::resp_body_string(resp)
      doc <- xml2::read_html(html_content)
      
      # Find all links
      links <- xml2::xml_find_all(doc, "//a[@href]")
      hrefs <- xml2::xml_attr(links, "href")
      
      # Filter for change log links (Excel or PDF files with ChangeLog in name)
      changelog_candidates <- grep(
        paste0(".*", prefix, ".*[Cc]hange.*[Ll]og.*\\.(xlsx|pdf)"),
        hrefs,
        ignore.case = TRUE,
        value = TRUE
      )
      
      if (length(changelog_candidates) > 0) {
        # If interactive and user might want to choose, collect all change logs
        # Otherwise, match change logs where the requested version appears
        if (interactive()) {
          # In interactive mode, collect all change logs for this type
          # User can choose which one they want
          matching_links <- changelog_candidates
        } else {
          # Non-interactive: match change logs where the requested version appears
          # Change logs are typically in range format: DXCCSR-ChangeLog-v20251-v20261.xlsx
          # The requested version (e.g., v2026.1) should match the end of the range (v20261)
          matching_links <- character()
          
          for (link in changelog_candidates) {
            # Check if the version (in compact format) appears at the end of the filename
            # This handles range formats like v20251-v20261 where v2026.1 -> v20261
            if (grepl(paste0("-", version_compact, "\\.(xlsx|pdf)$"), link, ignore.case = TRUE)) {
              matching_links <- c(matching_links, link)
            }
            # Also check for exact version match anywhere in filename
            else if (grepl(version_compact, link, ignore.case = TRUE)) {
              matching_links <- c(matching_links, link)
            }
            # Also check for version with dots/hyphens
            else if (grepl(gsub("\\.", "-", version), link, ignore.case = TRUE)) {
              matching_links <- c(matching_links, link)
            }
          }
        }
        
        if (length(matching_links) > 0) {
          # Make all URLs absolute
          for (link in matching_links) {
            if (!grepl("^https?://", link)) {
              if (grepl("^/", link)) {
                absolute_url <- paste0("https://hcup-us.ahrq.gov", link)
              } else {
                absolute_url <- paste0(base_url, link)
              }
            } else {
              absolute_url <- link
            }
            changelog_urls <- c(changelog_urls, absolute_url)
          }
          # Remove duplicates
          changelog_urls <- unique(changelog_urls)
        }
      }
    }, error = function(e) {
      # Continue to next URL
    })
  }
  
  # Always try direct file patterns to find change logs that might not be on archive page
  # This ensures we find the latest change logs even if they're not yet linked on the archive page
  # Also try range formats (e.g., v20251-v20261 for v2026.1)
  # We do this even if we found some URLs, to ensure we have all available options
  # Generate possible file patterns based on known naming conventions
  # Format: DXCCSR-ChangeLog-v20251-v20261.xlsx (range) or DXCCSR-ChangeLog-v20261.xlsx (single version)
  version_url <- gsub("\\.", "-", version)
  
  # For range formats, we need to find the previous version
  # Change logs show changes FROM previous version TO requested version
  # e.g., DXCCSR-ChangeLog-v20251-v20261.xlsx shows changes from v2025.1 to v2026.1
  # So if requesting v2026.1, we look for a change log ending with v20261
  # Extract year and minor version to construct previous version
  version_num <- gsub("[^0-9]", "", version)
  if (nchar(version_num) >= 5) {
    year <- as.numeric(substr(version_num, 1, 4))
    minor <- as.numeric(substr(version_num, 5, nchar(version_num)))
    
    # Try previous version for range format
    if (minor > 1) {
      prev_version_compact <- paste0("v", year, sprintf("%02d", minor - 1))
    } else if (year > 2019) {
      # Previous year, last minor version (assume .1, but could check)
      prev_version_compact <- paste0("v", year - 1, "1")
    } else {
      prev_version_compact <- NULL
    }
  } else {
    prev_version_compact <- NULL
  }
  
  file_patterns <- c(
    # Range format Excel files (preferred): DXCCSR-ChangeLog-v20251-v20261.xlsx
    # This shows changes TO the requested version
    if (!is.null(prev_version_compact)) {
      paste0(prefix, "-ChangeLog-", prev_version_compact, "-", version_compact, ".xlsx")
    } else NULL,
    # Also try with different separators
    if (!is.null(prev_version_compact)) {
      paste0(prefix, "_ChangeLog_", prev_version_compact, "_", version_compact, ".xlsx")
    } else NULL,
    # Single version Excel files (if they exist)
    paste0(prefix, "-ChangeLog-", version_compact, ".xlsx"),
    paste0(prefix, "-ChangeLog-", version_url, ".xlsx"),
    paste0(prefix, "_ChangeLog_", version_compact, ".xlsx"),
    paste0(prefix, "_ChangeLog_", version_url, ".xlsx"),
    # PDF files
    paste0(prefix, "-", version_url, "_ChangeLog.pdf"),
    paste0(prefix, "_", version_url, "_ChangeLog.pdf"),
    paste0(prefix, "-", version_url, "-ChangeLog.pdf"),
    paste0(prefix, "_", version_url, "-ChangeLog.pdf")
  )
  
  # Remove NULL entries
  file_patterns <- file_patterns[!sapply(file_patterns, is.null)]
  
  for (pattern in file_patterns) {
    test_url <- paste0(base_url, pattern)
    
    # Test if URL exists with a HEAD request
    url_exists <- tryCatch({
      resp <- httr2::request(test_url) |>
        httr2::req_method("HEAD") |>
        httr2::req_user_agent("HCUPtools R package") |>
        httr2::req_timeout(5) |>
        httr2::req_perform()
      
      status <- httr2::resp_status(resp)
      status %in% c(200, 302, 301, 303)
    }, error = function(e) FALSE)
    
    if (url_exists) {
      changelog_urls <- c(changelog_urls, test_url)
      # If we found a match and we're not in interactive mode, we can break early
      # In interactive mode, collect all possible options
      if (!interactive() && length(changelog_urls) > 0) {
        break
      }
    }
  }
  # Remove duplicates
  changelog_urls <- unique(changelog_urls)
  
  # If multiple change logs found, let user choose interactively
  changelog_url <- NULL
  if (length(changelog_urls) > 0 && interactive()) {
    # In interactive mode, always show menu if there are multiple options
    # If only one, still show it so user can see what was selected
    if (length(changelog_urls) > 1) {
      changelog_url <- select_changelog_interactive(changelog_urls, version, type)
    } else {
      # Only one option, but show it to user and confirm
      message("Found 1 change log file. Using: ", basename(changelog_urls[1]))
      changelog_url <- changelog_urls[1]
    }
  } else if (length(changelog_urls) == 1) {
    # Non-interactive: use the single match
    changelog_url <- changelog_urls[1]
  } else if (length(changelog_urls) > 1) {
    # Non-interactive: use first (prefer Excel)
    excel_urls <- grep("\\.xlsx", changelog_urls, ignore.case = TRUE, value = TRUE)
    if (length(excel_urls) > 0) {
      changelog_url <- excel_urls[1]
    } else {
      changelog_url <- changelog_urls[1]
    }
  }
  
  # Return based on format
  if (format == "url") {
    if (is.null(changelog_url)) {
      warning("Could not determine change log URL. Returning generic URL.")
      return(paste0(base_url, "ccsr_archive.jsp"))
    }
    return(changelog_url)
  }
  
  # Helper function to download the change log file (PDF or Excel)
  download_changelog_file <- function(url) {
    if (is.null(url)) {
      stop("Could not locate change log file for this version. ",
           "Change logs may not be available for all versions. ",
           "Try visiting the HCUP CCSR archive page: ",
           "https://hcup-us.ahrq.gov/toolssoftware/ccsr/ccsr_archive.jsp")
    }
    
    # Download to cache directory
    cache_dir <- file.path(tempdir(), "HCUPtools_cache")
    if (!dir.exists(cache_dir)) {
      dir.create(cache_dir, recursive = TRUE)
    }
    
    dest_file <- file.path(cache_dir, basename(url))
    
    # Check if already cached
    if (file.exists(dest_file)) {
      message("Using cached change log: ", dest_file)
      return(dest_file)
    }
    
    tryCatch({
      resp <- httr2::request(url) |>
        httr2::req_user_agent("HCUPtools R package") |>
        httr2::req_timeout(60) |>
        httr2::req_perform()
      
      httr2::resp_body_raw(resp) |>
        writeBin(dest_file)
      
      message("Change log downloaded to: ", dest_file)
      return(dest_file)
    }, error = function(e) {
      stop("Failed to download change log: ", conditionMessage(e),
           "\nTry accessing the URL directly: ", url)
    })
  }
  
  if (format == "download") {
    return(download_changelog_file(changelog_url))
  }
  
  if (format == "view") {
    if (is.null(changelog_url)) {
      stop("Could not locate change log file for this version. ",
           "Change logs may not be available for all versions. ",
           "Try visiting the HCUP CCSR archive page: ",
           "https://hcup-us.ahrq.gov/toolssoftware/ccsr/ccsr_archive.jsp")
    }
    
    changelog_file <- download_changelog_file(changelog_url)
    
    # Determine file type and open appropriately
    is_pdf <- grepl("\\.pdf$", changelog_file, ignore.case = TRUE)
    is_excel <- grepl("\\.xlsx?$", changelog_file, ignore.case = TRUE)
    
    if (is_pdf) {
      # Open PDF in default viewer
      if (.Platform$OS.type == "windows") {
        shell.exec(changelog_file)
      } else if (Sys.info()["sysname"] == "Darwin") {
        system(paste("open", shQuote(changelog_file)))
      } else {
        # Linux
        system(paste("xdg-open", shQuote(changelog_file)))
      }
      message("Change log PDF opened in default viewer")
    } else if (is_excel) {
      # Open Excel file in default application
      if (.Platform$OS.type == "windows") {
        shell.exec(changelog_file)
      } else if (Sys.info()["sysname"] == "Darwin") {
        system(paste("open", shQuote(changelog_file)))
      } else {
        # Linux
        system(paste("xdg-open", shQuote(changelog_file)))
      }
      message("Change log Excel file opened in default application")
    } else {
      # Unknown file type, try to open anyway
      if (.Platform$OS.type == "windows") {
        shell.exec(changelog_file)
      } else if (Sys.info()["sysname"] == "Darwin") {
        system(paste("open", shQuote(changelog_file)))
      } else {
        system(paste("xdg-open", shQuote(changelog_file)))
      }
      message("Change log file opened in default application")
    }
    
    return(invisible(changelog_file))
  }
  
  if (format == "extract") {
    if (is.null(changelog_url)) {
      stop("Could not locate change log file for this version. ",
           "Change logs may not be available for all versions. ",
           "Try visiting the HCUP CCSR archive page: ",
           "https://hcup-us.ahrq.gov/toolssoftware/ccsr/ccsr_archive.jsp")
    }
    
    changelog_file <- download_changelog_file(changelog_url)
    
    # Determine file type
    is_pdf <- grepl("\\.pdf$", changelog_file, ignore.case = TRUE)
    is_excel <- grepl("\\.xlsx?$", changelog_file, ignore.case = TRUE)
    
    if (is_pdf) {
      # Check if pdftools is available
      if (!requireNamespace("pdftools", quietly = TRUE)) {
        stop("Package 'pdftools' is required for PDF text extraction. ",
             "Install it with: install.packages('pdftools')\n",
             "Alternatively, use format = 'view' to open the file, or format = 'download' to save it.")
      }
      
      tryCatch({
        # Extract text from PDF
        pdf_text <- pdftools::pdf_text(changelog_file)
        extracted_text <- paste(pdf_text, collapse = "\n\n")
        
        message("Text extracted from change log PDF")
        return(extracted_text)
      }, error = function(e) {
        stop("Failed to extract text from PDF: ", conditionMessage(e),
             "\nThe PDF may be image-based or corrupted. ",
             "Try format = 'view' to open it, or format = 'download' to save it.")
      })
    } else if (is_excel) {
      # For Excel files, read and convert to text representation
      if (!requireNamespace("readxl", quietly = TRUE)) {
        stop("Package 'readxl' is required for Excel file extraction. ",
             "Install it with: install.packages('readxl')\n",
             "Alternatively, use format = 'view' to open the file, or format = 'download' to save it.")
      }
      
      tryCatch({
        # Read all sheets from Excel file
        sheet_names <- readxl::excel_sheets(changelog_file)
        extracted_text <- character()
        
        for (sheet in sheet_names) {
          sheet_data <- readxl::read_excel(changelog_file, sheet = sheet, .name_repair = "minimal")
          extracted_text <- c(extracted_text, 
                             paste0("\n=== Sheet: ", sheet, " ===\n"),
                             utils::capture.output(print(sheet_data)))
        }
        
        result <- paste(extracted_text, collapse = "\n")
        message("Content extracted from change log Excel file (", length(sheet_names), " sheet(s))")
        return(result)
      }, error = function(e) {
        stop("Failed to extract content from Excel file: ", conditionMessage(e),
             "\nTry format = 'view' to open it, or format = 'download' to save it.")
      })
    } else {
      stop("Unsupported file type for text extraction. ",
           "Only PDF and Excel files are supported. ",
           "Use format = 'view' to open the file, or format = 'download' to save it.")
    }
  }
  
  if (format == "read") {
    if (is.null(changelog_url)) {
      stop("Could not locate change log file for this version. ",
           "Change logs may not be available for all versions. ",
           "Try visiting the HCUP CCSR archive page: ",
           "https://hcup-us.ahrq.gov/toolssoftware/ccsr/ccsr_archive.jsp")
    }
    
    changelog_file <- download_changelog_file(changelog_url)
    
    # Check if it's an Excel file
    is_excel <- grepl("\\.xlsx?$", changelog_file, ignore.case = TRUE)
    if (!is_excel) {
      stop("The 'read' format only works with Excel files. ",
           "This change log is a PDF file. ",
           "Use format = 'view' to open it, format = 'download' to save it, ",
           "or format = 'extract' to extract text.")
    }
    
    # Check if readxl is available
    if (!requireNamespace("readxl", quietly = TRUE)) {
      stop("Package 'readxl' is required to read Excel files. ",
           "Install it with: install.packages('readxl')\n",
           "Alternatively, use format = 'view' to open the file, or format = 'download' to save it.")
    }
    
    # Handle as_data_table parameter
    if (is.null(as_data_table)) {
      # Check if data.table is available
      has_data_table <- requireNamespace("data.table", quietly = TRUE)
      
      if (interactive() && has_data_table && requested_version == "latest") {
        cat("\nWould you like to import as a data.table? (faster for large datasets)\n")
        cat("  [1] Yes (data.table)\n")
        cat("  [2] No (tibble/data.frame) - default\n")
        cat("Enter choice (1 or 2, or press Enter for default): ")
        
        choice <- readline()
        choice <- trimws(choice)
        
        as_data_table <- (choice == "1" || tolower(choice) == "y" || tolower(choice) == "yes")
      } else {
        as_data_table <- FALSE
      }
    }
    
    tryCatch({
      # Read the first sheet (change logs typically have one main sheet)
      sheet_names <- readxl::excel_sheets(changelog_file)
      
      if (length(sheet_names) == 0) {
        stop("No sheets found in the Excel file.")
      }
      
      # Read the first sheet (or let user choose if multiple)
      if (length(sheet_names) == 1) {
        data <- readxl::read_excel(changelog_file, sheet = 1, .name_repair = "minimal")
      } else if (interactive() && requested_version == "latest") {
        # Multiple sheets - let user choose
        cat("\n=== Available Sheets ===\n")
        for (i in seq_along(sheet_names)) {
          cat(sprintf("%2d. %s\n", i, sheet_names[i]))
        }
        cat("\nSelect a sheet (enter number, or press Enter for first sheet): ")
        selection <- readline()
        selection <- trimws(selection)
        
        if (selection == "" || is.na(as.integer(selection))) {
          sheet_idx <- 1
        } else {
          sheet_idx <- as.integer(selection)
          if (is.na(sheet_idx) || sheet_idx < 1 || sheet_idx > length(sheet_names)) {
            warning("Invalid selection. Using first sheet.")
            sheet_idx <- 1
          }
        }
        data <- readxl::read_excel(changelog_file, sheet = sheet_idx, .name_repair = "minimal")
      } else {
        # Non-interactive: use first sheet
        data <- readxl::read_excel(changelog_file, sheet = 1, .name_repair = "minimal")
      }
      
      # Convert to data.table if requested
      if (as_data_table) {
        data <- data.table::as.data.table(data)
      }
      
      message("Change log read successfully (", nrow(data), " rows, ", ncol(data), " columns)")
      return(data)
    }, error = function(e) {
      stop("Failed to read change log Excel file: ", conditionMessage(e),
           "\nTry format = 'view' to open it, or format = 'download' to save it.")
    })
  }
  
  # Format == "text"
  if (is.null(changelog_url)) {
    return(paste0(
      "Change log for CCSR ", type, " version ", version, ".\n",
      "Could not locate change log file for this version.\n",
      "Please visit the HCUP CCSR archive page for change log information:\n",
      "https://hcup-us.ahrq.gov/toolssoftware/ccsr/ccsr_archive.jsp\n\n",
      "Note: Change logs may not be available for all versions, or may be ",
      "located on different pages. Try checking the archive page directly."
    ))
  }
  
  # For text format, return URL and instructions
  file_type <- if (grepl("\\.xlsx?$", changelog_url, ignore.case = TRUE)) {
    "Excel file"
  } else if (grepl("\\.pdf$", changelog_url, ignore.case = TRUE)) {
    "PDF file"
  } else {
    "file"
  }
  
  return(paste0(
    "Change log for CCSR ", type, " version ", version, ".\n",
    "URL: ", changelog_url, "\n",
    "File type: ", file_type, "\n\n",
    "To view the change log:\n",
    "  - Use format = 'view' to open in default application\n",
    "  - Use format = 'download' to save the file\n",
    "  - Use format = 'extract' to extract text/content\n",
    "  - Or visit the URL directly in your browser"
  ))
}

#' Interactive selection of change log file
#' @noRd
select_changelog_interactive <- function(changelog_urls, version, type) {
  if (length(changelog_urls) == 0) {
    return(NULL)
  }
  
  if (length(changelog_urls) == 1) {
    return(changelog_urls[1])
  }
  
  # Parse file names to extract version information
  file_info <- lapply(changelog_urls, function(url) {
    filename <- basename(url)
    
    # Extract version range from filename (e.g., v20251-v20261)
    version_match <- regmatches(filename, regexpr("v\\d+-v\\d+", filename))
    if (length(version_match) > 0) {
      version_range <- version_match[1]
      # Extract the end version for sorting (e.g., v20261 from v20251-v20261)
      end_version <- regmatches(version_range, regexpr("v\\d+$", version_range))
      if (length(end_version) > 0) {
        # Convert to numeric for sorting (e.g., v20261 -> 20261)
        version_num <- as.numeric(gsub("v", "", end_version[1]))
      } else {
        version_num <- 0
      }
    } else {
      # Try single version
      version_match <- regmatches(filename, regexpr("v\\d+", filename))
      if (length(version_match) > 0) {
        version_range <- version_match[1]
        version_num <- as.numeric(gsub("v", "", version_range))
      } else {
        version_range <- "Unknown"
        version_num <- 0
      }
    }
    
    # Determine file type
    file_type <- if (grepl("\\.xlsx", filename, ignore.case = TRUE)) {
      "Excel"
    } else if (grepl("\\.pdf", filename, ignore.case = TRUE)) {
      "PDF"
    } else {
      "Unknown"
    }
    
    list(
      url = url,
      filename = filename,
      version_range = version_range,
      file_type = file_type,
      version_num = version_num
    )
  })
  
  # Sort by version number (most recent first - descending order)
  version_nums <- sapply(file_info, function(x) x$version_num)
  file_info <- file_info[order(version_nums, decreasing = TRUE)]
  
  # Show interactive menu
  cat("\n=== Available Change Log Files ===\n\n")
  for (i in seq_along(file_info)) {
    info <- file_info[[i]]
    cat(sprintf("%2d. %s\n", i, info$filename))
    cat(sprintf("    Version range: %s | Type: %s\n", 
                info$version_range, info$file_type))
    cat("\n")
  }
  
  cat("Select a change log file (enter number): ")
  selection <- readline()
  selection <- as.integer(selection)
  
  if (is.na(selection) || selection < 1 || selection > length(file_info)) {
    stop("Invalid selection. Please run the function again and select a valid number.")
  }
  
  return(file_info[[selection]]$url)
}

#' Download HCUP Summary Trend Tables
#'
#' Downloads HCUP Summary Trend Tables from the HCUP website. These tables provide
#' information on hospital utilization derived from HCUP databases, including trends
#' in inpatient and emergency department utilization.
#'
#' @param table_id Character string or numeric specifying which table to download.
#'   Can be:
#'   - A table number (e.g., "1", "2a", "6b", "11c") for specific tables
#'   - "all" to download all available tables as a ZIP file
#'   - NULL (default) to show an interactive menu for selecting a table (in interactive sessions),
#'     or return a list of available tables (in non-interactive sessions)
#' @param dest_dir Character string specifying the destination directory for the
#'   downloaded file(s). If NULL (default), files are saved to a temporary directory.
#' @param cache Logical. If TRUE (default), downloaded files are cached to avoid
#'   re-downloading on subsequent calls.
#'
#' @return If `table_id` is NULL and session is non-interactive, returns a data frame listing available tables.
#'   Otherwise, returns the path(s) to the downloaded file(s).
#'
#' @details
#' The HCUP Summary Trend Tables include information on:
#' - Overview of trends in inpatient and emergency department utilization
#' - All inpatient encounter types
#' - Inpatient encounter types (normal newborns, deliveries, elective/non-elective stays)
#' - Inpatient service lines (maternal/neonatal, mental health, injuries, surgeries, etc.)
#' - ED treat-and-release visits
#'
#' Each table is available as an Excel file with state-specific, region-specific,
#' and national statistics.
#'
#' The function automatically discovers available tables by scraping the HCUP website,
#' so it will automatically adapt to new tables or version changes.
#'
#' For more information, see:
#' https://hcup-us.ahrq.gov/reports/trendtables/summarytrendtables.jsp
#'
#' @examples
#' \donttest{
#' # List available tables
#' available_tables <- download_trend_tables()
#' print(available_tables)
#'
#' # Download a specific table
#' table_path <- download_trend_tables("2a")
#'
#' # Bulk ZIP (\code{table_id = "all"}) is only available when HCUP publishes a
#' # combined file; otherwise use individual IDs from the listing above.
#' }
#'
#' @importFrom httr2 request req_timeout req_user_agent req_perform resp_body_raw resp_body_string
#' @importFrom utils unzip
#' @importFrom tibble tibble
#' @importFrom xml2 read_html xml_find_all xml_attr xml_text
#' @importFrom dplyr mutate arrange
#' @importFrom rlang .data
#' @export
download_trend_tables <- function(table_id = NULL,
                                  dest_dir = NULL,
                                  cache = TRUE) {
  base_url <- "https://hcup-us.ahrq.gov/reports/trendtables/"
  
  # Get available tables dynamically from HCUP website
  available_tables <- get_available_trend_tables(base_url, cache)
  
  # If table_id is NULL, show interactive menu or return list
  if (is.null(table_id)) {
    if (interactive() && nrow(available_tables) > 0) {
      # Show interactive menu
      table_id <- select_trend_table_interactive(available_tables)
      if (is.null(table_id)) {
        # User cancelled or invalid selection
        return(available_tables)
      }
    } else {
      # Non-interactive: return list of available tables
      return(available_tables)
    }
  }
  
  # Convert to character and lowercase
  table_id <- tolower(as.character(table_id))
  
  # Handle "all" case - download ZIP file
  if (table_id == "all") {
    # Try to find the ZIP file URL dynamically
    zip_urls <- c(
      "https://hcup-us.ahrq.gov/reports/trendtables/HCUP_SummaryTrendTables_All.zip",
      "https://hcup-us.ahrq.gov/reports/trendtables/HCUP_SummaryTrendTables.zip"
    )
    
    # Test which URL exists
    zip_url <- NULL
    for (test_url in zip_urls) {
      url_exists <- tryCatch({
        resp <- httr2::request(test_url) |>
          httr2::req_method("HEAD") |>
          httr2::req_user_agent("HCUPtools R package") |>
          httr2::req_timeout(5) |>
          httr2::req_perform()
        status <- httr2::resp_status(resp)
        status %in% c(200, 302, 301, 303)
      }, error = function(e) FALSE)
      
      if (url_exists) {
        zip_url <- test_url
        break
      }
    }
    
    if (is.null(zip_url)) {
      # ZIP file not available - offer to download all individual tables instead
      if (interactive()) {
        cat("\n")
        cat("The 'all tables' ZIP file is not available on the HCUP website.\n")
        cat("Would you like to download all individual tables instead?\n")
        cat("  [1] Yes - Download all ", nrow(available_tables), " tables\n", sep = "")
        cat("  [2] No - Cancel\n")
        cat("Enter choice (1 or 2, or press Enter to cancel): ")
        
        choice <- readline()
        choice <- trimws(choice)
        
        if (choice == "1" || tolower(choice) == "y" || tolower(choice) == "yes") {
          # Download all individual tables
          message("Downloading all ", nrow(available_tables), " individual tables...")
          message("This may take several minutes.")
          
          downloaded_files <- character()
          for (i in seq_len(nrow(available_tables))) {
            table_info <- available_tables[i, ]
            cat(sprintf("Downloading table %d/%d: %s...\n", 
                       i, nrow(available_tables), table_info$table_name))
            
            tryCatch({
              file_path <- download_trend_tables(
                table_id = table_info$table_id,
                dest_dir = dest_dir,
                cache = cache
              )
              downloaded_files <- c(downloaded_files, file_path)
            }, error = function(e) {
              warning("Failed to download table ", table_info$table_id, ": ", 
                     conditionMessage(e))
            })
          }
          
          message("\nDownload complete! ", length(downloaded_files), 
                 " table(s) downloaded successfully.")
          return(downloaded_files)
        } else {
          stop("Download cancelled.")
        }
      } else {
        stop("The 'all tables' ZIP file is not available on the HCUP website. ",
             "Please download individual tables using their table IDs. ",
             "Use download_trend_tables() to see available tables.")
      }
    }
    
    file_name <- basename(zip_url)
    
    if (is.null(dest_dir)) {
      dest_dir <- tempdir()
    }
    
    dest_file <- file.path(dest_dir, file_name)
    
    # Check cache if enabled
    if (cache && file.exists(dest_file)) {
      message("Using cached file: ", dest_file)
      return(dest_file)
    }
    
    message("Downloading all HCUP Summary Trend Tables...")
    message("This may take several minutes due to file size (~81 MB)")
    
    tryCatch({
      resp <- httr2::request(zip_url) |>
        httr2::req_timeout(300) |>
        httr2::req_user_agent("HCUPtools R package") |>
        httr2::req_perform()
      
      httr2::resp_body_raw(resp) |>
        writeBin(dest_file)
      
      message("Download complete: ", dest_file)
      return(dest_file)
    }, error = function(e) {
      stop("Failed to download trend tables: ", conditionMessage(e),
           "\nThe ZIP file may not be available. ",
           "Try downloading individual tables instead.")
    })
  }
  
  # Validate table_id
  if (!table_id %in% available_tables$table_id) {
    stop("Invalid table_id. Use download_trend_tables() to see available tables.")
  }
  
  # Get file name from available_tables (handles different naming patterns)
  table_info <- available_tables[available_tables$table_id == table_id, ]
  file_name <- table_info$file_name[1]
  url <- paste0(base_url, file_name)
  
  # Set destination directory
  if (is.null(dest_dir)) {
    dest_dir <- tempdir()
  }
  
  dest_file <- file.path(dest_dir, file_name)
  
  # Check cache if enabled
  if (cache && file.exists(dest_file)) {
    message("Using cached file: ", dest_file)
    return(dest_file)
  }
  
  # Get table name for message
  table_name <- table_info$table_name[1]
  message("Downloading: ", table_name)
  message("URL: ", url)
  
  tryCatch({
    resp <- httr2::request(url) |>
      httr2::req_timeout(120) |>
      httr2::req_user_agent("HCUPtools R package") |>
      httr2::req_perform()
    
    httr2::resp_body_raw(resp) |>
      writeBin(dest_file)
    
    message("Download complete: ", dest_file)
    return(dest_file)
  }, error = function(e) {
    stop("Failed to download table ", table_id, ": ", conditionMessage(e))
  })
}

#' Fetch available trend tables from HCUP website
#'
#' @param base_url Character string, base URL for trend tables
#' @param cache Logical, whether to use cached results
#' @return A tibble with table_id, table_name, and file_name columns
#'
#' @noRd
get_available_trend_tables <- function(base_url, cache = TRUE) {
  # Cache key for storing results
  cache_name <- "hcup_trend_tables_list"
  cache_path <- file.path(tempdir(), cache_name)
  
  # Check cache (valid for 24 hours)
  cached_tables <- NULL
  if (cache && file.exists(cache_path)) {
    cache_info <- file.info(cache_path)
    cache_age <- as.numeric(Sys.time() - cache_info$mtime, units = "hours")
    if (cache_age < 24) {
      tryCatch({
        cached_data <- readRDS(cache_path)
        if (inherits(cached_data, "data.frame") && 
            "table_id" %in% names(cached_data) && 
            "table_name" %in% names(cached_data)) {
          cached_tables <- cached_data
        }
      }, error = function(e) {
        # Cache file corrupted, will re-fetch
      })
    }
  }
  
  # If not cached, fetch from HCUP website
  if (is.null(cached_tables)) {
    url <- "https://hcup-us.ahrq.gov/reports/trendtables/summarytrendtables.jsp"
    
    tryCatch({
      resp <- httr2::request(url) |>
        httr2::req_user_agent("HCUPtools R package") |>
        httr2::req_timeout(30) |>
        httr2::req_perform()
      
      html_content <- httr2::resp_body_string(resp)
      doc <- xml2::read_html(html_content)
      
      # Find all links to Excel files
      links <- xml2::xml_find_all(doc, "//a[@href]")
      hrefs <- xml2::xml_attr(links, "href")
      link_texts <- xml2::xml_text(links)
      
      # Filter for trend table links (Excel files)
      trend_table_pattern <- "HCUP_SummaryTrendTables_T.*\\.xlsx"
      trend_table_links <- grep(trend_table_pattern, hrefs, value = TRUE, ignore.case = TRUE)
      trend_table_texts <- link_texts[grep(trend_table_pattern, hrefs, ignore.case = TRUE)]
      
      # Extract table IDs and names
      tables_list <- list()
      
      for (i in seq_along(trend_table_links)) {
        href <- trend_table_links[i]
        text <- trimws(trend_table_texts[i])
        
        # Extract table ID from filename (e.g., "T2a" -> "2a")
        table_id_match <- regmatches(href, regexpr("T([0-9]+[a-z]?)", href, ignore.case = TRUE))
        if (length(table_id_match) > 0) {
          table_id <- tolower(sub("T", "", table_id_match, ignore.case = TRUE))
          
          # Use link text as table name, or construct from filename if text is empty
          if (nchar(text) == 0 || text == href) {
            # Try to extract from href or use a generic name
            table_name <- paste("Table", toupper(table_id))
          } else {
            # Clean up table name - remove "Table X." prefix if present
            table_name <- gsub("^Table\\s+[0-9]+[a-z]?\\.\\s*", "", text, ignore.case = TRUE)
            table_name <- trimws(table_name)
            # If cleaning removed everything, use original text
            if (nchar(table_name) == 0) {
              table_name <- text
            }
          }
          
          tables_list[[length(tables_list) + 1]] <- list(
            table_id = table_id,
            table_name = table_name,
            file_name = basename(href)
          )
        }
      }
      
      # Also check for the "all" ZIP file
      zip_pattern <- "HCUP_SummaryTrendTables.*\\.zip"
      zip_links <- grep(zip_pattern, hrefs, value = TRUE, ignore.case = TRUE)
      if (length(zip_links) > 0) {
        # Use the first ZIP file found (usually the "all" file)
        zip_file <- basename(zip_links[1])
      } else {
        zip_file <- "HCUP_SummaryTrendTables_All.zip"
      }
      
      if (length(tables_list) > 0) {
        cached_tables <- dplyr::bind_rows(tables_list) |>
          dplyr::mutate(
            # Sort by numeric part first, then alphabetic
            sort_key = as.numeric(gsub("[^0-9]", "", .data$table_id)),
            sort_key2 = gsub("[0-9]", "", .data$table_id)
          ) |>
          dplyr::arrange(.data$sort_key, .data$sort_key2) |>
          dplyr::select(-.data$sort_key, -.data$sort_key2)
        
        # Cache the result
        tryCatch({
          saveRDS(cached_tables, cache_path)
        }, error = function(e) {
          # Cache save failed, continue without caching
        })
      } else {
        # Fallback: return empty tibble with expected structure
        cached_tables <- tibble::tibble(
          table_id = character(0),
          table_name = character(0),
          file_name = character(0)
        )
      }
    }, error = function(e) {
      warning("Could not fetch trend tables from HCUP website: ", 
              conditionMessage(e),
              ". Using fallback method.")
      # Fallback: return empty tibble
      cached_tables <- tibble::tibble(
        table_id = character(0),
        table_name = character(0),
        file_name = character(0)
      )
    })
  }
  
  return(cached_tables)
}

#' Interactive selection of trend table
#' @noRd
select_trend_table_interactive <- function(available_tables) {
  if (nrow(available_tables) == 0) {
    stop("No trend tables available.")
  }
  
  if (nrow(available_tables) == 1) {
    return(available_tables$table_id[1])
  }
  
  # Show interactive menu
  cat("\n=== Available HCUP Summary Trend Tables ===\n\n")
  for (i in seq_len(nrow(available_tables))) {
    table <- available_tables[i, ]
    cat(sprintf("%2d. [%s] %s\n", 
                i, 
                table$table_id,
                table$table_name))
  }
  cat("\n")
  cat("  [0] Download all tables (ZIP file - if available)\n")
  cat("\n")
  cat("Select a table (enter number, or 0 for all): ")
  
  selection <- readline()
  selection <- trimws(selection)
  
  if (selection == "" || is.na(as.integer(selection))) {
    return(NULL)
  }
  
  selection <- as.integer(selection)
  
  if (selection == 0) {
    return("all")
  }
  
  if (is.na(selection) || selection < 1 || selection > nrow(available_tables)) {
    stop("Invalid selection. Please run the function again and select a valid number.")
  }
  
  return(available_tables$table_id[selection])
}

#' HCUPtools: Access and Work with HCUP Resources and Datasets
#'
#' @description
#' `HCUPtools` is a comprehensive R package for accessing and working with 
#' publicly available resources from the Agency for Healthcare Research and 
#' Quality (AHRQ) Healthcare Cost and Utilization Project (HCUP). The package 
#' provides streamlined access to HCUP's Clinical Classifications Software 
#' Refined (CCSR) mapping files and Summary Trend Tables, enabling researchers 
#' and analysts to efficiently map ICD-10 codes to CCSR categories and access 
#' HCUP statistical reports.
#'
#' @details
#' The package provides functions to:
#' \itemize{
#' \item Download CCSR mapping files directly from the HCUP website
#' \item Map ICD-10-CM diagnosis codes and ICD-10-PCS procedure codes to CCSR categories
#' \item Access CCSR category descriptions and metadata
#' \item Download HCUP Summary Trend Tables
#' \item Read downloaded files from disk (ZIP, CSV, Excel, or directories)
#' \item Manage multiple CCSR versions and change logs
#' \item Generate proper AHRQ/HCUP citations
#' }
#'
#' The package does **not** redistribute CCSR data files but facilitates 
#' direct download from the official AHRQ HCUP website, ensuring users 
#' always have access to the latest versions and maintain compliance with 
#' HCUP data use policies.
#' 
#' **Important:** This package only accesses publicly available and free HCUP 
#' resources (CCSR tools and Summary Trend Tables). It does NOT access any 
#' HCUP databases (NIS, KID, SID, NEDS, etc.) that require purchase through 
#' the HCUP Central Distributor.
#'
#' For more information about CCSR, see the 
#' [official HCUP CCSR overview page](https://hcup-us.ahrq.gov/toolssoftware/ccsr/ccs_refined.jsp).
#'
#' @section Main Functions:
#' 
#' **CCSR Mapping Functions:**
#' \itemize{
#' \item `download_ccsr()` - Download CCSR mapping files from HCUP website
#' \item `read_ccsr()` - Read CCSR mapping files from disk (ZIP, CSV, Excel, or directory)
#' \item `ccsr_map()` - Map ICD-10 codes to CCSR categories (long/wide/default formats)
#' \item `get_ccsr_description()` - Get clinical descriptions for CCSR codes
#' \item `list_ccsr_versions()` - List available CCSR versions
#' \item `ccsr_changelog()` - Get CCSR change log for a specific version
#' }
#' 
#' **HCUP Summary Trend Tables Functions:**
#' \itemize{
#' \item `download_trend_tables()` - Download HCUP Summary Trend Tables
#' \item `read_trend_table()` - Read HCUP Summary Trend Table Excel files from disk
#' \item `list_trend_table_sheets()` - List available sheets in a trend table file
#' }
#' 
#' **Utility Functions:**
#' \itemize{
#' \item `hcup_citation()` - Generate citations for HCUP resources (CCSR or Trend Tables)
#' }
#'
#' @section Key Features:
#' \itemize{
#' \item **Direct Download**: Automatically download CCSR mapping files and Summary Trend Tables from HCUP
#' \item **Multiple Formats**: Support for long, wide, and default-only output formats
#' \item **Cross-Classification**: Handle one-to-many mappings (multiple CCSR categories per ICD-10 code)
#' \item **Version Management**: Access multiple CCSR versions and change logs
#' \item **Citation Generation**: Automatically generate proper AHRQ/HCUP citations
#' \item **File Reading**: Read downloaded files from disk (ZIP, CSV, Excel, or directories)
#' \item **Caching**: Intelligent caching to avoid redundant downloads
#' \item **Interactive Menus**: User-friendly interactive selection for files and options
#' }
#'
#' @section Legal and Usage:
#' **Important Disclaimer:** This package is an independent, non-commercial tool 
#' developed by a third party. It is **not affiliated with, endorsed by, or 
#' supported by AHRQ or HCUP in any way.** This package is not an official AHRQ 
#' or HCUP product. Users are responsible for ensuring compliance with all 
#' applicable HCUP Data Use Agreements (DUAs).
#'
#' **User Responsibilities:**
#' \itemize{
#' \item Understanding and complying with all applicable HCUP data usage policies
#' \item Verifying the accuracy of results
#' \item Citing the appropriate AHRQ/HCUP sources in publications
#' \item Ensuring compliance with all HCUP Data Use Agreements
#' }
#'
#' For official HCUP information and policies, visit:
#' \itemize{
#' \item [HCUP Data Use Agreement Training](https://hcup-us.ahrq.gov/tech_assist/dua.jsp)
#' \item [HCUP Data Use Agreements](https://hcup-us.ahrq.gov/team/NationwideDUA.pdf)
#' \item [HCUP Publishing Requirements](https://hcup-us.ahrq.gov/db/publishing.jsp)
#' \item [CCSR Overview](https://hcup-us.ahrq.gov/toolssoftware/ccsr/ccs_refined.jsp)
#' \item [HCUP Summary Trend Tables](https://hcup-us.ahrq.gov/reports/trendtables/summarytrendtables.jsp)
#' }
#'
#' @section Technical Details:
#' \itemize{
#' \item **Encoding**: Files are read with UTF-8 encoding to handle special characters
#' \item **Leading Zeros**: ICD-10 codes are preserved as character strings
#' \item **Caching**: Downloaded files are cached by default to avoid redundant downloads
#' \item **Cross-Classification**: Handles one-to-many mappings (multiple CCSR categories per ICD-10 code)
#' \item **Version Management**: Automatic detection of latest versions and support for historical versions
#' }
#'
#' @seealso
#' Useful links:
#' \itemize{
#' \item \url{https://github.com/vikrant31/HCUPtools} - Package GitHub repository
#' \item \url{https://hcup-us.ahrq.gov/toolssoftware/ccsr/ccs_refined.jsp} - CCSR Overview
#' \item \url{https://hcup-us.ahrq.gov/toolssoftware/ccsr/ccs_refined.jsp} - CCSR Tools and Downloads
#' \item \url{https://hcup-us.ahrq.gov/reports/trendtables/summarytrendtables.jsp} - Summary Trend Tables
#' \item \url{https://hcup-us.ahrq.gov/} - HCUP Homepage
#' }
#'
#' @author
#' **Maintainer:** Vikrant Dev Rathore \email{rathore.vikrant@gmail.com}
#'
#' @keywords internal
"_PACKAGE"

#' Read CCSR Mapping Files from Disk
#'
#' Reads previously downloaded CCSR mapping files from disk. If no file path is
#' provided, automatically finds and reads cached files from `download_ccsr()`.
#'
#' @param file_path Optional character string, path to a CCSR mapping file. Can be:
#'   - A ZIP file path (will be extracted and read)
#'   - A CSV/Excel file path (will be read directly)
#'   - A directory path containing extracted CCSR files
#'   If NULL (default), automatically searches the cache directory for files
#'   downloaded via `download_ccsr()`.
#' @param type Character string specifying the type of CCSR file. Must be one
#'   of: "diagnosis" (or "dx") for ICD-10-CM diagnosis codes, or "procedure"
#'   (or "pr") for ICD-10-PCS procedure codes. If NULL and `file_path` is NULL,
#'   defaults to "diagnosis". If `file_path` is provided, the function will
#'   attempt to infer the type from the file name or contents.
#' @param version Character string specifying the CCSR version to read from cache.
#'   Use "latest" (default) to read the most recent version, or specify a version
#'   like "v2026.1", "v2025.1", etc. Only used when `file_path` is NULL.
#' @param clean_names Logical. If TRUE (default), column names are cleaned to
#'   follow R naming conventions (snake_case).
#' @param as_data_table Logical or NULL. If TRUE and the `data.table` package is
#'   available, returns a `data.table` object instead of a tibble. If FALSE,
#'   returns a tibble. If NULL (default), prompts the user interactively to
#'   choose (only in interactive sessions). In non-interactive sessions, defaults
#'   to FALSE. Note: tibbles are already data frames and work with all standard
#'   R data frame operations.
#' @param name Optional character string, suggested variable name for the
#'   returned data. This is only used for display/messaging purposes and does
#'   not automatically assign the data to a variable. You must still assign the
#'   result: `my_data <- read_ccsr()`. If NULL (default), a name is suggested
#'   based on the file type and version.
#'
#' @return A tibble (or data.table if `as_data_table = TRUE`) containing the
#'   CCSR mapping data. Tibbles are data frames and can be used with all
#'   standard R data frame operations, including `dplyr`, `data.table`, and
#'   base R functions.
#'
#' @note To use the data, assign it to a variable:
#'   `my_data <- read_ccsr()`. The `name` parameter is only for display
#'   purposes and does not automatically assign the data.
#'
#' @details
#' This function can read CCSR files in several formats:
#' - ZIP files downloaded from HCUP (will extract and read the CSV/Excel file)
#' - CSV files (extracted from ZIP or saved separately)
#' - Excel files (if `readxl` package is available)
#' - Directories containing extracted files
#' - Cached files from `download_ccsr()` (automatic if `file_path` is NULL)
#'
#' The function automatically detects the file format and handles encoding
#' issues, preserving leading zeros in ICD-10 codes.
#'
#' When `file_path` is NULL, the function automatically searches the cache
#' directory (`tempdir()/HCUPtools_cache/`) for files matching the specified
#' `type` and `version`. This makes it easy to read previously downloaded
#' files without needing to know the exact file path.
#'
#' @examples
#' \donttest{
#' # Populate cache, then read (requires network)
#' invisible(download_ccsr("diagnosis"))
#' dx_map <- read_ccsr(as_data_table = FALSE)
#' invisible(download_ccsr("procedure"))
#' pr_map <- read_ccsr(type = "procedure", as_data_table = FALSE)
#'
#' # From a local file on your machine (uncomment and set the path):
#' # dx_map <- read_ccsr("/path/to/DXCCSR-v2026-1.zip", as_data_table = FALSE)
#' # dx_map <- read_ccsr("/path/to/DXCCSR_v2026_1.csv", as_data_table = FALSE)
#' # dx_map <- read_ccsr("/path/to/extracted_ccsr_files/", as_data_table = FALSE)
#'
#' head(dx_map)
#' nrow(dx_map)
#' }
#'
#' @importFrom readr read_csv cols locale
#' @importFrom tibble as_tibble
#' @importFrom utils unzip
#' @importFrom readxl read_excel
#' @export
read_ccsr <- function(file_path = NULL,
                     type = NULL,
                     version = "latest",
                     clean_names = TRUE,
                     as_data_table = NULL,
                     name = NULL) {
  
  # Determine suggested variable name early (estimate based on type/version)
  # We'll refine it later once we have the actual file_path
  suggested_name <- determine_suggested_name(name, type, version, file_path)
  
  # Handle interactive prompt for as_data_table FIRST (before file selection)
  # Show the tip along with the data.table question
  if (is.null(as_data_table)) {
    as_data_table <- prompt_for_data_table(suggested_name, "read_ccsr")
  }
  
  # If no file_path provided, automatically find cached file or show interactive menu
  if (is.null(file_path)) {
    cached_files <- list_cached_ccsr_files(type)
    
    if (length(cached_files) == 0) {
      stop("No cached CCSR file found. ",
           "Please download a file first using `download_ccsr()` ",
           "or provide a `file_path`.")
    }
    
    # If only one file, use it automatically
    if (length(cached_files) == 1) {
      file_path <- cached_files[[1]]$path
      message("Reading cached file: ", basename(file_path))
    } else {
      # Multiple files - show interactive menu
      file_path <- select_cached_ccsr_file_interactive(cached_files, type, version)
    }
  }
  
  # Validate file path
  if (!file.exists(file_path)) {
    stop("File or directory not found: ", file_path)
  }
  
  # Determine if it's a file or directory
  is_dir <- dir.exists(file_path)
  is_zip <- grepl("\\.zip$", file_path, ignore.case = TRUE)
  is_csv <- grepl("\\.csv$", file_path, ignore.case = TRUE)
  is_xlsx <- grepl("\\.xlsx?$", file_path, ignore.case = TRUE)
  
  # Update suggested variable name now that we have the actual file_path
  suggested_name <- determine_suggested_name(name, type, version, file_path)
  
  # Handle ZIP files
  if (is_zip) {
    result <- read_ccsr_from_zip(file_path, type, clean_names)
    result <- convert_to_data_table_if_requested(result, as_data_table)
    show_assignment_message(suggested_name, result)
    return(result)
  }
  
  # Handle directories
  if (is_dir) {
    result <- read_ccsr_from_dir(file_path, type, clean_names)
    result <- convert_to_data_table_if_requested(result, as_data_table)
    show_assignment_message(suggested_name, result)
    return(result)
  }
  
  # Handle CSV files
  if (is_csv) {
    result <- read_ccsr_from_csv(file_path, type, clean_names)
    result <- convert_to_data_table_if_requested(result, as_data_table)
    show_assignment_message(suggested_name, result)
    return(result)
  }
  
  # Handle Excel files
  if (is_xlsx) {
    if (!requireNamespace("readxl", quietly = TRUE)) {
      stop("Package 'readxl' is required for reading Excel files. ",
           "Install it with: install.packages('readxl')")
    }
    result <- read_ccsr_from_excel(file_path, type, clean_names)
    result <- convert_to_data_table_if_requested(result, as_data_table)
    show_assignment_message(suggested_name, result)
    return(result)
  }
  
  stop("Unsupported file format. Expected ZIP, CSV, Excel, or directory.")
}

#' Read CCSR from ZIP file
#' @noRd
read_ccsr_from_zip <- function(zip_path, type, clean_names) {
  # Create temporary extraction directory
  extract_dir <- tempfile("ccsr_extract_")
  dir.create(extract_dir, showWarnings = FALSE)
  on.exit(unlink(extract_dir, recursive = TRUE), add = TRUE)
  
  # Extract ZIP file
  utils::unzip(zip_path, exdir = extract_dir)
  
  # Find CSV or Excel files in extracted directory
  files <- list.files(extract_dir, full.names = TRUE, recursive = TRUE)
  csv_files <- files[grepl("\\.csv$", files, ignore.case = TRUE)]
  xlsx_files <- files[grepl("\\.xlsx?$", files, ignore.case = TRUE)]
  
  # Prefer CSV, fall back to Excel
  if (length(csv_files) > 0) {
    # Use the first CSV file (or try to find the right one based on type)
    target_file <- select_ccsr_file(csv_files, type)
    return(read_ccsr_from_csv(target_file, type, clean_names))
  } else if (length(xlsx_files) > 0) {
    if (!requireNamespace("readxl", quietly = TRUE)) {
      stop("Package 'readxl' is required for reading Excel files. ",
           "Install it with: install.packages('readxl')")
    }
    target_file <- select_ccsr_file(xlsx_files, type)
    return(read_ccsr_from_excel(target_file, type, clean_names))
  } else {
    stop("No CSV or Excel files found in ZIP archive")
  }
}

#' Read CCSR from directory
#' @noRd
read_ccsr_from_dir <- function(dir_path, type, clean_names) {
  # Find CSV or Excel files
  files <- list.files(dir_path, full.names = TRUE, recursive = TRUE)
  csv_files <- files[grepl("\\.csv$", files, ignore.case = TRUE)]
  xlsx_files <- files[grepl("\\.xlsx?$", files, ignore.case = TRUE)]
  
  if (length(csv_files) > 0) {
    target_file <- select_ccsr_file(csv_files, type)
    return(read_ccsr_from_csv(target_file, type, clean_names))
  } else if (length(xlsx_files) > 0) {
    if (!requireNamespace("readxl", quietly = TRUE)) {
      stop("Package 'readxl' is required for reading Excel files. ",
           "Install it with: install.packages('readxl')")
    }
    target_file <- select_ccsr_file(xlsx_files, type)
    return(read_ccsr_from_excel(target_file, type, clean_names))
  } else {
    stop("No CSV or Excel files found in directory")
  }
}

#' Read CCSR from CSV file
#' @noRd
read_ccsr_from_csv <- function(csv_path, type, clean_names) {
  # Try different encodings
  encodings <- c("UTF-8", "latin1", "ISO-8859-1")
  data <- NULL
  
  for (enc in encodings) {
    tryCatch({
      data <- readr::read_csv(
        csv_path,
        locale = readr::locale(encoding = enc),
        col_types = readr::cols(.default = "c"),
        show_col_types = FALSE,
        progress = FALSE
      )
      break
    }, error = function(e) {
      # Try next encoding
    })
  }
  
  if (is.null(data)) {
    stop("Failed to read CSV file. Tried encodings: ", paste(encodings, collapse = ", "))
  }
  
  # Convert to tibble
  data <- tibble::as_tibble(data)
  data <- normalize_ccsr_character_columns(data)
  
  # Clean column names if requested
  if (clean_names) {
    names(data) <- gsub("\\s+", "_", names(data))
    names(data) <- gsub("[^A-Za-z0-9_]", "", names(data))
    names(data) <- tolower(names(data))
    names(data) <- gsub("_+", "_", names(data))
    names(data) <- gsub("^_|_$", "", names(data))
  }
  
  # Preserve leading zeros in ICD codes
  # Infer type if not provided
  if (is.null(type)) {
    col_names <- tolower(names(data))
    if (any(grepl("dxccsr|diagnosis|icd.*10.*cm", col_names))) {
      type <- "diagnosis"
    } else if (any(grepl("prccsr|procedure|icd.*10.*pcs", col_names))) {
      type <- "procedure"
    } else {
      type <- "diagnosis"  # Default
    }
  }
  
  # Find ICD column
  col_names <- tolower(names(data))
  if (type == "diagnosis") {
    patterns <- c("icd.*10.*cm", "icd.*10", "diagnosis.*code", "^code$", "^dx$")
  } else {
    patterns <- c("icd.*10.*pcs", "icd.*10", "procedure.*code", "^code$", "^pr$")
  }
  
  icd_col <- NULL
  for (pattern in patterns) {
    match <- grep(pattern, col_names, value = TRUE)
    if (length(match) > 0) {
      icd_col <- match[1]
      break
    }
  }
  
  if (!is.null(icd_col) && icd_col %in% names(data)) {
    # Format ICD codes (preserve leading zeros)
    data[[icd_col]] <- format_icd_codes(data[[icd_col]])
  }
  
  return(data)
}

#' Read CCSR from Excel file
#' @noRd
read_ccsr_from_excel <- function(excel_path, type, clean_names) {
  # Read first sheet
  data <- readxl::read_excel(excel_path, sheet = 1)
  
  # Convert to tibble
  data <- tibble::as_tibble(data)
  data <- normalize_ccsr_character_columns(data)
  
  # Clean column names if requested
  if (clean_names) {
    names(data) <- gsub("\\s+", "_", names(data))
    names(data) <- gsub("[^A-Za-z0-9_]", "", names(data))
    names(data) <- tolower(names(data))
    names(data) <- gsub("_+", "_", names(data))
    names(data) <- gsub("^_|_$", "", names(data))
  }
  
  # Preserve leading zeros in ICD codes
  # Infer type if not provided
  if (is.null(type)) {
    col_names <- tolower(names(data))
    if (any(grepl("dxccsr|diagnosis|icd.*10.*cm", col_names))) {
      type <- "diagnosis"
    } else if (any(grepl("prccsr|procedure|icd.*10.*pcs", col_names))) {
      type <- "procedure"
    } else {
      type <- "diagnosis"  # Default
    }
  }
  
  # Find ICD column
  col_names <- tolower(names(data))
  if (type == "diagnosis") {
    patterns <- c("icd.*10.*cm", "icd.*10", "diagnosis.*code", "^code$", "^dx$")
  } else {
    patterns <- c("icd.*10.*pcs", "icd.*10", "procedure.*code", "^code$", "^pr$")
  }
  
  icd_col <- NULL
  for (pattern in patterns) {
    match <- grep(pattern, col_names, value = TRUE)
    if (length(match) > 0) {
      icd_col <- match[1]
      break
    }
  }
  
  if (!is.null(icd_col) && icd_col %in% names(data)) {
    # Format ICD codes (preserve leading zeros)
    data[[icd_col]] <- format_icd_codes(data[[icd_col]])
  }
  
  return(data)
}

#' Select appropriate CCSR file from multiple files
#' @noRd
select_ccsr_file <- function(files, type) {
  if (length(files) == 1) {
    return(files[1])
  }
  
  # If type is specified, try to match
  if (!is.null(type)) {
    type <- tolower(type)
    if (type %in% c("dx", "diagnosis")) {
      pattern <- "dx|diagnosis"
    } else if (type %in% c("pr", "procedure")) {
      pattern <- "pr|procedure"
    } else {
      pattern <- NULL
    }
    
    if (!is.null(pattern)) {
      matches <- grep(pattern, files, ignore.case = TRUE, value = TRUE)
      if (length(matches) > 0) {
        return(matches[1])
      }
    }
  }
  
  # Default to first file
  return(files[1])
}


#' List all cached CCSR files
#' @noRd
list_cached_ccsr_files <- function(type = NULL) {
  cache_dir <- file.path(tempdir(), "HCUPtools_cache")
  if (!dir.exists(cache_dir)) {
    return(list())
  }
  
  all_files <- list.files(cache_dir, full.names = TRUE, pattern = "\\.zip$", ignore.case = TRUE)
  
  if (length(all_files) == 0) {
    return(list())
  }
  
  # Parse file information
  files_info <- list()
  for (file in all_files) {
    basename_file <- basename(file)
    
    # Determine type
    file_type <- if (grepl("DXCCSR", basename_file, ignore.case = TRUE)) {
      "diagnosis"
    } else if (grepl("PRCCSR", basename_file, ignore.case = TRUE)) {
      "procedure"
    } else {
      "unknown"
    }
    
    # Extract version
    version_match <- regmatches(basename_file, 
                               regexpr("v\\d{4}[-.]\\d+", basename_file, 
                                      ignore.case = TRUE))
    version <- if (length(version_match) > 0) version_match[1] else "unknown"
    
    # Filter by type if specified
    if (!is.null(type)) {
      type <- tolower(type)
      if (type %in% c("dx", "diagnosis") && file_type != "diagnosis") {
        next
      } else if (type %in% c("pr", "procedure") && file_type != "procedure") {
        next
      }
    }
    
    files_info[[length(files_info) + 1]] <- list(
      path = file,
      type = file_type,
      version = version,
      basename = basename_file
    )
  }
  
  return(files_info)
}

#' @noRd
select_cached_ccsr_file_interactive <- function(cached_files, type = NULL, version = "latest") {
  # Filter by version if specified
  if (version != "latest") {
    version_url <- gsub("\\.", "-", version)
    version_pattern <- paste0("v", gsub("v", "", version_url))
    version_pattern_alt <- gsub("-", "_", version_pattern)
    
    matching_version <- sapply(cached_files, function(f) {
      grepl(paste0(version_pattern, "|", version_pattern_alt), f$version, ignore.case = TRUE)
    })
    
    if (any(matching_version)) {
      cached_files <- cached_files[matching_version]
    }
  }
  
  # If only one after filtering, return it
  if (length(cached_files) == 1) {
    return(cached_files[[1]]$path)
  }
  
  # Show interactive menu
  cat("\n=== Available Cached CCSR Files ===\n\n")
  for (i in seq_along(cached_files)) {
    f <- cached_files[[i]]
    cat(sprintf("%2d. %s (%s, %s)\n", 
                i, 
                f$basename, 
                tools::toTitleCase(f$type),
                f$version))
  }
  cat("\n")
  
  # Get user selection
  if (interactive()) {
    selection <- readline("Select a file (enter number): ")
    selection <- as.integer(selection)
    
    if (is.na(selection) || selection < 1 || selection > length(cached_files)) {
      stop("Invalid selection. Please run the function again and select a valid number.")
    }
    
    return(cached_files[[selection]]$path)
  } else {
    # Non-interactive: return latest or first
    if (version == "latest") {
      # Sort by version and return latest
      versions <- sapply(cached_files, function(f) f$version)
      version_nums <- gsub("v", "", versions)
      version_nums <- gsub("-", ".", version_nums)
      version_parts <- strsplit(version_nums, "\\.")
      years <- as.numeric(sapply(version_parts, function(x) x[1]))
      minors <- as.numeric(sapply(version_parts, function(x) x[2]))
      sorted_idx <- order(years, minors, decreasing = TRUE)
      return(cached_files[[sorted_idx[1]]]$path)
    } else {
      return(cached_files[[1]]$path)
    }
  }
}

#' Prompt user for data.table preference (interactive only)
#' @noRd
prompt_for_data_table <- function(suggested_name = NULL, func_name = "read_ccsr") {
  # Check if data.table is available
  has_data_table <- requireNamespace("data.table", quietly = TRUE)
  
  if (!interactive()) {
    # Non-interactive: default to FALSE (tibble)
    return(FALSE)
  }
  
  if (!has_data_table) {
    # data.table not available: return FALSE
    return(FALSE)
  }
  
  # Interactive session with data.table available: ask user
  cat("\n")
  
  # Show tip about assigning to variable along with the data.table question
  if (!is.null(suggested_name)) {
    cat("Tip: Assign the result to a variable to use the data:\n")
    cat("  ", suggested_name, " <- ", func_name, "(...)\n", sep = "")
    cat("\n")
  }
  
  cat("Would you like to import as a data.table? (faster for large datasets)\n")
  cat("  [1] Yes (data.table)\n")
  cat("  [2] No (tibble/data.frame) - default\n")
  cat("Enter choice (1 or 2, or press Enter for default): ")
  
  choice <- readline()
  choice <- trimws(choice)
  
  if (choice == "1" || tolower(choice) == "y" || tolower(choice) == "yes") {
    return(TRUE)
  } else {
    return(FALSE)
  }
}

#' @noRd
convert_to_data_table_if_requested <- function(data, as_data_table) {
  if (as_data_table) {
    if (requireNamespace("data.table", quietly = TRUE)) {
      return(data.table::as.data.table(data))
    } else {
      warning("Package 'data.table' is not available. Returning tibble instead. ",
              "Install it with: install.packages('data.table')")
      return(data)
    }
  }
  return(data)
}


#' Determine suggested variable name for the data
#' @noRd
determine_suggested_name <- function(name, type, version, file_path) {
  if (!is.null(name) && is.character(name) && length(name) == 1 && nchar(name) > 0) {
    # User provided a name, use it
    return(name)
  }
  
  # Generate suggested name based on type and version
  type_abbrev <- if (is.null(type)) {
    "ccsr"
  } else if (tolower(type) %in% c("dx", "diagnosis")) {
    "dx_map"
  } else if (tolower(type) %in% c("pr", "procedure")) {
    "pr_map"
  } else {
    "ccsr_map"
  }
  
  # Add version if available
  if (!is.null(version) && version != "latest") {
    version_clean <- gsub("[^0-9]", "", version)
    if (nchar(version_clean) > 0) {
      type_abbrev <- paste0(type_abbrev, "_v", version_clean)
    }
  }
  
  return(type_abbrev)
}

#' Show message about how to assign the data to a variable
#' @noRd
show_assignment_message <- function(suggested_name, data) {
  if (interactive() && !is.null(suggested_name)) {
    data_type <- if (inherits(data, "data.table")) {
      "data.table"
    } else if (inherits(data, "tbl_df")) {
      "tibble"
    } else {
      "data.frame"
    }
    
    n_rows <- nrow(data)
    n_cols <- ncol(data)
    
    cat("\n")
    cat("=== Data Imported Successfully ===\n")
    cat("Type: ", data_type, "\n", sep = "")
    cat("Dimensions: ", n_rows, " rows x ", n_cols, " columns\n", sep = "")
    cat("\n")
    cat("To use this data, assign it to a variable:\n")
    cat("  ", suggested_name, " <- read_ccsr(...)\n", sep = "")
    cat("\n")
    cat("Then you can use it:\n")
    cat("  head(", suggested_name, ")\n", sep = "")
    cat("  nrow(", suggested_name, ")\n", sep = "")
    cat("  # Or use with dplyr, data.table, etc.\n")
    cat("\n")
  }
}

---
title: "Getting Started with HCUPtools"
author: "Vikrant Dev Rathore"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Getting Started with HCUPtools}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE,
  fig.width = 7,
  fig.height = 5
)
```

`HCUPtools` is an R package for accessing and working with resources from the **Agency for Healthcare Research and Quality (AHRQ) Healthcare Cost and Utilization Project (HCUP)**. This vignette provides a comprehensive guide to using the package for common healthcare data analysis tasks.

## Installation and Setup

```{r, eval = FALSE}
# Install from CRAN (evaluated only when you knit locally; skipped during R CMD check)
install.packages("HCUPtools")

# Load the package
library(HCUPtools)
library(dplyr)  # For data manipulation examples
```

## Part 1: Downloading CCSR Mapping Files

The Clinical Classifications Software Refined (CCSR) is a tool developed by AHRQ/HCUP to categorize ICD-10-CM diagnosis codes and ICD-10-PCS procedure codes into clinically meaningful categories. The `download_ccsr()` function provides direct access to these mapping files.

### Download Latest Version

```{r, eval=FALSE}
# Download the latest diagnosis CCSR mapping file
dx_map <- download_ccsr("diagnosis")

# Download the latest procedure CCSR mapping file
pr_map <- download_ccsr("procedure")
```

### Download Specific Version

```{r, eval=FALSE}
# Download a specific version (useful for reproducibility)
dx_map_v2025 <- download_ccsr("diagnosis", version = "v2025.1")
pr_map_v2025 <- download_ccsr("procedure", version = "v2025.1")
```

### List Available Versions

```{r, eval=FALSE}
# List all available versions
all_versions <- list_ccsr_versions()
print(all_versions)

# List only diagnosis versions
dx_versions <- list_ccsr_versions("diagnosis")

# List only procedure versions
pr_versions <- list_ccsr_versions("procedure")
```

## Part 2: Mapping ICD-10 Codes to CCSR Categories

Once you have downloaded a mapping file, you can use `ccsr_map()` to map ICD-10 codes to CCSR categories. This function supports multiple output formats to accommodate different analytical needs.

### Prepare Sample Data

```{r, eval=FALSE}
# Create sample patient data with ICD-10 diagnosis codes
patient_data <- tibble::tibble(
  patient_id = 1:10,
  admission_date = as.Date(c("2024-01-15", "2024-02-20", "2024-03-10", 
                              "2024-04-05", "2024-05-12", "2024-06-18",
                              "2024-07-22", "2024-08-30", "2024-09-14",
                              "2024-10-08")),
  icd10_dx = c("E11.9", "I10", "M79.3", "E78.5", "K21.9", 
               "I50.9", "N18.6", "E78.5", "I25.10", "J44.1")
)
```

### Long Format (Default)

The long format duplicates records for each assigned CCSR category. This is essential for cross-classification analysis where you need to count all assigned categories.

```{r, eval=FALSE}
# Map codes using long format (default)
mapped_long <- ccsr_map(
  data = patient_data,
  code_col = "icd10_dx",
  map_df = dx_map,
  output_format = "long"
)

# View the results
head(mapped_long, 20)

# Count occurrences of each CCSR category
ccsr_counts <- mapped_long |>
  count(ccsr_category, sort = TRUE)
print(ccsr_counts)
```

**Use Case**: Long format is ideal when you want to:
- Count how many times each CCSR category appears
- Analyze cross-classifications (one ICD-10 code mapping to multiple CCSR categories)
- Create frequency tables of CCSR categories

### Wide Format

The wide format creates multiple columns (CCSR_1, CCSR_2, etc.) for multiple categories, keeping one row per ICD-10 code.

```{r, eval=FALSE}
# Map codes using wide format
mapped_wide <- ccsr_map(
  data = patient_data,
  code_col = "icd10_dx",
  map_df = dx_map,
  output_format = "wide"
)

# View the results
head(mapped_wide)
```

**Use Case**: Wide format is ideal when you want to:
- Keep all CCSR category slots on one row per **input diagnosis code row**
- Then aggregate to patient or encounter level with your own rules (e.g., principal diagnosis)
- Maintain the original data structure with additional CCSR columns

### Default Category Only

For diagnosis codes, CCSR assigns a "default" category that is recommended for principal diagnosis analysis. Use `default_only = TRUE` to extract only this default category.

```{r, eval=FALSE}
# Map codes using default category only
mapped_default <- ccsr_map(
  data = patient_data,
  code_col = "icd10_dx",
  map_df = dx_map,
  default_only = TRUE
)

# View the results
head(mapped_default)
```

**Use Case**: Default category is ideal when you want to:
- Analyze principal diagnoses only
- Follow HCUP recommendations for diagnosis analysis
- Maintain one-to-one mapping (one ICD-10 code = one CCSR category)

## Part 3: Getting CCSR Descriptions

To understand what CCSR categories mean, use `get_ccsr_description()`:

```{r, eval=FALSE}
# Get descriptions for specific CCSR codes
ccsr_codes <- c("ADM010", "NEP003", "CIR019", "END001", "MBD001")
descriptions <- get_ccsr_description(ccsr_codes, map_df = dx_map)
print(descriptions)

# Get descriptions without pre-downloaded mapping (will download automatically)
descriptions_auto <- get_ccsr_description(
  c("ADM010", "NEP003"), 
  type = "diagnosis"
)
```

## Part 4: Working with Procedure Codes

The package also supports ICD-10-PCS procedure codes:

```{r, eval=FALSE}
# Download procedure mapping
pr_map <- download_ccsr("procedure")

# Create sample procedure data
procedure_data <- tibble::tibble(
  case_id = 1:5,
  procedure_date = as.Date(c("2024-01-20", "2024-02-15", "2024-03-22",
                              "2024-04-10", "2024-05-18")),
  icd10_pcs = c("0DB60ZZ", "0DT70ZZ", "0WQ3XZ", "0FB00ZZ", "0HB00ZX")
)

# Map procedure codes
mapped_procedures <- ccsr_map(
  data = procedure_data,
  code_col = "icd10_pcs",
  map_df = pr_map
)

# View the results
head(mapped_procedures)
```

## Part 5: Complete Analysis Workflow

Here's a complete workflow for analyzing CCSR categories in a dataset:

```{r, eval=FALSE}
# Step 1: Download mapping file
dx_map <- download_ccsr("diagnosis")

# Step 2: Map diagnosis codes
patient_data_mapped <- ccsr_map(
  data = patient_data,
  code_col = "icd10_dx",
  map_df = dx_map,
  output_format = "long"
)

# Step 3: Count occurrences of each CCSR category
ccsr_counts <- patient_data_mapped |>
  count(ccsr_category, sort = TRUE)

# Step 4: Merge with descriptions for reporting
ccsr_counts_with_desc <- ccsr_counts |>
  left_join(
    get_ccsr_description(
      unique(patient_data_mapped$ccsr_category), 
      map_df = dx_map
    ),
    by = c("ccsr_category" = "ccsr_code")
  )

# Step 5: View the final results
print(ccsr_counts_with_desc)
```

## Part 6: Downloading HCUP Summary Trend Tables

The package also provides access to HCUP Summary Trend Tables, which contain aggregated information on hospital utilization trends:

```{r, eval=FALSE}
# List available tables (interactive menu)
available_tables <- download_trend_tables()
print(available_tables)

# Download a specific table by ID
# Table 2a: All Inpatient Encounter Types - Trends in Number of Discharges
table_path <- download_trend_tables("2a")

# Download all tables as a ZIP file (~81 MB)
all_tables_zip <- download_trend_tables("all")
```

The trend tables include:
- Overview of trends in inpatient and emergency department utilization
- All inpatient encounter types (discharges, percent, length of stay, mortality, population rates)
- Inpatient encounter types (normal newborns, deliveries, elective/non-elective stays)
- Inpatient service lines (maternal/neonatal, mental health, injuries, surgeries, medical conditions)
- ED treat-and-release visits

For more information, see: [HCUP Summary Trend Tables](https://hcup-us.ahrq.gov/reports/trendtables/summarytrendtables.jsp)

### Reading Trend Tables

```{r, eval=FALSE}
# Read the trend table data
trend_data <- read_trend_table(table_path, sheet = "National")
head(trend_data)

# List available sheets
sheets <- list_trend_table_sheets(table_path)
print(sheets)

# Read specific state data
california_data <- read_trend_table(table_path, sheet = "California")
```

## Part 7: Accessing CCSR Change Logs

View changes between CCSR versions:

```{r, eval=FALSE}
# Get change log as data table (default)
changelog <- ccsr_changelog(version = "v2026.1")
print(changelog)

# Get change log URL
changelog_url <- ccsr_changelog(version = "v2026.1", format = "url")

# View change log in default PDF viewer
ccsr_changelog(version = "v2026.1", format = "view")

# Download change log file
changelog_file <- ccsr_changelog(version = "v2026.1", format = "download")
```

## Part 8: Generating Citations

When using HCUP data in publications, always cite the source properly:

```{r, eval=FALSE}
# Generate text citation for CCSR
cat(hcup_citation())

# Generate citation for Summary Trend Tables
cat(hcup_citation(resource = "trend_tables"))

# Generate BibTeX citation (for LaTeX documents)
cat(hcup_citation(format = "bibtex"))

# Generate R citation object (for R markdown)
citation_obj <- hcup_citation(format = "r")
print(citation_obj)
```

## Part 9: Reading Downloaded Files

If you've already downloaded files, you can read them directly:

```{r, eval=FALSE}
# Read CCSR file from various formats
dx_map <- read_ccsr("path/to/DXCCSR-v2026-1.zip")
dx_map <- read_ccsr("path/to/DXCCSR_v2026-1.csv")
dx_map <- read_ccsr("path/to/DXCCSR_v2026-1.xlsx")
dx_map <- read_ccsr("path/to/extracted_directory/")

# Read trend table Excel file
national_data <- read_trend_table(
  "path/to/HCUP_SummaryTrendTables_T2a.xlsx",
  sheet = "National"
)
```

## Important Notes

### Data Download
- The package downloads data directly from HCUP, so an internet connection is required for the first download
- Downloaded files are cached by default to avoid re-downloading
- Set `cache = FALSE` to disable caching

### Cross-Classification
- One ICD-10 code can map to multiple CCSR categories
- Use long format to see all mappings
- Use default category for principal diagnosis analysis

### Default Categories
- For diagnosis codes, CCSR assigns a default category recommended for principal diagnosis analysis
- Use `default_only = TRUE` to extract only the default category

### Performance
- CCSR mapping files contain ~75,000 rows
- Consider using `as_data_table = TRUE` in `read_ccsr()` and `read_trend_table()` for very large datasets

## Legal and Compliance

**Important Disclaimer:** This package is an independent, non-commercial tool developed by a third party. It is **not affiliated with, endorsed by, or supported by AHRQ or HCUP in any way.** This package is not an official AHRQ or HCUP product.

This package facilitates access to **publicly available and free** HCUP resources:

- **CCSR Mapping Files** - Classification software tools (free download)
- **HCUP Summary Trend Tables** - Aggregated statistical reports (free download)

**Critical:** This package does **NOT** access any HCUP databases (NIS, KID, SID, NEDS, etc.) that require purchase through the HCUP Central Distributor.

### User Responsibilities

Users are responsible for:
- Ensuring compliance with all applicable HCUP Data Use Agreements (DUAs)
- Verifying the accuracy of results
- Citing the appropriate AHRQ/HCUP sources in publications
- Understanding and adhering to all HCUP data usage restrictions

### Essential Resources

- [HCUP Data Use Agreement Training](https://hcup-us.ahrq.gov/tech_assist/dua.jsp)
- [HCUP Data Use Agreements](https://hcup-us.ahrq.gov/team/NationwideDUA.pdf)
- [HCUP Publishing Requirements](https://hcup-us.ahrq.gov/db/publishing.jsp)
- [CCSR Overview](https://hcup-us.ahrq.gov/toolssoftware/ccsr/ccs_refined.jsp)
- [HCUP Summary Trend Tables](https://hcup-us.ahrq.gov/reports/trendtables/summarytrendtables.jsp)

## Additional Resources

- **Package GitHub**: https://github.com/vikrant31/HCUPtools
- **HCUP Homepage**: https://hcup-us.ahrq.gov/
- **CCSR Overview**: https://hcup-us.ahrq.gov/toolssoftware/ccsr/ccs_refined.jsp
- **HCUP CCSR Tools**: https://hcup-us.ahrq.gov/toolssoftware/ccsr/ccs_refined.jsp
- **HCUP Summary Trend Tables**: https://hcup-us.ahrq.gov/reports/trendtables/summarytrendtables.jsp

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ccsr_map.R
\name{ccsr_map}
\alias{ccsr_map}
\title{Map ICD-10 Codes to CCSR Categories}
\usage{
ccsr_map(
  data,
  code_col,
  map_df,
  type = NULL,
  default_only = FALSE,
  output_format = "long",
  keep_all = TRUE
)
}
\arguments{
\item{data}{A data frame or tibble containing ICD-10 codes to be mapped.}

\item{code_col}{Character string specifying the name of the column in \code{data}
that contains the ICD-10 codes.}

\item{map_df}{A tibble containing the CCSR mapping data, typically obtained
from \code{download_ccsr()}.}

\item{type}{Character string specifying the type of mapping. Must be one of:
"diagnosis" (or "dx") for ICD-10-CM codes, or "procedure" (or "pr") for
ICD-10-PCS codes. If NULL (default), the function will attempt to infer
the type from the mapping data frame.}

\item{default_only}{Logical. For diagnosis codes only, if TRUE, returns only
the default CCSR category (recommended for principal diagnosis analysis).
If FALSE (default), returns all assigned CCSR categories including
cross-classifications.}

\item{output_format}{Character string specifying the output format. Must be
one of: "long" (default) or "wide". "long" format duplicates records for
each assigned CCSR category. "wide" format creates multiple columns
(CCSR_1, CCSR_2, etc.) for multiple categories.}

\item{keep_all}{Logical. If TRUE (default), returns all original columns
from \code{data} plus the CCSR mapping columns. If FALSE, returns only the
ICD-10 code column and CCSR mapping columns.}
}
\value{
A tibble with the original data plus CCSR mapping columns. The
structure depends on \code{output_format}:
\itemize{
\item For "long" format: Each row represents one ICD-10 code and one CCSR
category assignment (rows are duplicated for multiple categories).
\item For "wide" format: Each row represents one ICD-10 code with multiple
CCSR category columns (CCSR_1, CCSR_2, etc.).
}
}
\description{
Maps ICD-10-CM diagnosis codes or ICD-10-PCS procedure codes to their
corresponding CCSR categories using a downloaded CCSR mapping file.
}
\details{
CCSR allows for cross-classification, meaning a single ICD-10 code can map
to multiple CCSR categories. The "long" format is recommended for analyses
where you want to count all assigned CCSR categories, while "wide" format
may be more convenient for patient-level analyses.

For diagnosis codes, CCSR also assigns a "default" category that is
recommended for principal diagnosis analysis. Use \code{default_only = TRUE} to
extract only this default category.
}
\examples{
\donttest{
# Download mapping file
dx_map <- download_ccsr("diagnosis")

# Create sample data
sample_data <- tibble::tibble(
  patient_id = 1:3,
  icd10_code = c("E11.9", "I10", "M79.3")
)

# Map codes (long format - default)
mapped_long <- ccsr_map(
  data = sample_data,
  code_col = "icd10_code",
  map_df = dx_map
)

# Map codes (wide format)
mapped_wide <- ccsr_map(
  data = sample_data,
  code_col = "icd10_code",
  map_df = dx_map,
  output_format = "wide"
)

# Map codes (default category only)
mapped_default <- ccsr_map(
  data = sample_data,
  code_col = "icd10_code",
  map_df = dx_map,
  default_only = TRUE
)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/download_ccsr.R
\name{download_ccsr}
\alias{download_ccsr}
\title{Download CCSR Mapping Files from HCUP}
\usage{
download_ccsr(
  type = "diagnosis",
  version = "latest",
  cache = TRUE,
  clean_names = TRUE
)
}
\arguments{
\item{type}{Character string specifying the type of CCSR file to download.
Must be one of: "diagnosis" (or "dx") for ICD-10-CM diagnosis codes, or
"procedure" (or "pr") for ICD-10-PCS procedure codes. Default is "diagnosis".}

\item{version}{Character string specifying the CCSR version to download.
Use "latest" to download the most recent version, or specify a version
like "v2026.1", "v2025.1", etc. Default is "latest".}

\item{cache}{Logical. If TRUE (default), the downloaded file is cached in
a temporary directory to avoid re-downloading on subsequent calls.}

\item{clean_names}{Logical. If TRUE (default), column names are cleaned to
follow R naming conventions (snake_case).}
}
\value{
A tibble containing the CCSR mapping data with the following columns:
\itemize{
\item For diagnosis files: ICD-10-CM code, CCSR category, default CCSR category,
and clinical descriptions
\item For procedure files: ICD-10-PCS code, CCSR category, and descriptions
}
}
\description{
Downloads and loads Clinical Classifications Software Refined (CCSR) mapping
files directly from the Agency for Healthcare Research and Quality (AHRQ)
Healthcare Cost and Utilization Project (HCUP) website.
}
\details{
This function downloads CCSR mapping files directly from the HCUP website.
The package does not redistribute these files but facilitates access to the
official AHRQ data sources.

The function handles:
\itemize{
\item Automatic URL construction based on type and version
\item ZIP file download and extraction
\item Proper encoding of special characters
\item Preservation of leading zeros in ICD-10 codes
\item Conversion to tidy tibble format
}
}
\examples{
\donttest{
# Download latest diagnosis CCSR mapping
dx_map <- download_ccsr("diagnosis")

# Download specific version of procedure CCSR mapping
pr_map <- download_ccsr("procedure", version = "v2025.1")

# Download without caching
dx_map <- download_ccsr("diagnosis", cache = FALSE)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_ccsr_description.R
\name{get_ccsr_description}
\alias{get_ccsr_description}
\title{Get CCSR Category Descriptions}
\usage{
get_ccsr_description(ccsr_codes, map_df = NULL, type = NULL)
}
\arguments{
\item{ccsr_codes}{Character vector of CCSR category codes (e.g., "ADM010",
"NEP003", "CIR019").}

\item{map_df}{Optional. A tibble containing CCSR mapping data with
descriptions. If provided, descriptions are extracted from this data frame.
If NULL (default), the function will attempt to download the latest mapping
file to extract descriptions.}

\item{type}{Character string specifying the type of CCSR codes. Must be one
of: "diagnosis" (or "dx") or "procedure" (or "pr"). If NULL (default), the
function will attempt to infer the type from the codes or mapping data.}
}
\value{
A tibble with columns:
\itemize{
\item \code{ccsr_code}: The CCSR category code
\item \code{description}: The full clinical description
\item Additional metadata columns if available in the mapping data
}
}
\description{
Retrieves the full clinical description for one or more CCSR category codes.
This function helps users interpret CCSR codes by providing their meaningful
clinical descriptions.
}
\details{
CCSR category codes follow specific naming conventions:
\itemize{
\item Diagnosis codes: Typically start with letters (e.g., "ADM010", "NEP003")
\item Procedure codes: Typically start with letters (e.g., "PRC001", "PRC002")
}

If a description is not found for a code, it will be marked as NA in the
result.
}
\examples{
\donttest{
# Get descriptions using downloaded mapping data
dx_map <- download_ccsr("diagnosis")
get_ccsr_description(c("ADM010", "NEP003", "CIR019"), map_df = dx_map)

# Get descriptions without pre-downloaded data (will download automatically)
get_ccsr_description(c("ADM010", "NEP003"), type = "diagnosis")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ccsr_changelog.R
\name{ccsr_changelog}
\alias{ccsr_changelog}
\title{Get CCSR Change Log}
\usage{
ccsr_changelog(
  version = "latest",
  type = "diagnosis",
  format = "read",
  as_data_table = NULL
)
}
\arguments{
\item{version}{Character string specifying the CCSR version. Use "latest"
(default) to get the change log for the most recent version, or specify a
version like "v2026.1", "v2025.1", etc.}

\item{type}{Character string specifying the type of CCSR. Must be one of:
"diagnosis" (or "dx") for ICD-10-CM diagnosis codes, or "procedure" (or
"pr") for ICD-10-PCS procedure codes. Default is "diagnosis".}

\item{format}{Character string specifying the output format. Options:
"read" (default) - Downloads and reads the Excel file as a data table/tibble (requires \code{readxl} package)
"text" - Returns change log information as text
"url" - Returns the URL to the change log document
"download" - Downloads and returns the change log file path
"view" - Downloads and opens the change log file in the default viewer
"extract" - Attempts to extract text from the file (requires \code{pdftools} for PDF or \code{readxl} for Excel)}

\item{as_data_table}{Logical. If TRUE, returns a \code{data.table} instead of a tibble.
Only used when \code{format = "read"}. If NULL (default), prompts the user interactively
to choose (only in interactive sessions). In non-interactive sessions, defaults to FALSE.}
}
\value{
Depending on \code{format}:
\itemize{
\item "read" (default): A tibble or data.table containing the change log data (if Excel file)
\item "text": Character string with change log information
\item "url": Character string with URL to change log
\item "download": Character string with path to downloaded file
\item "view": Opens the file and returns the file path (invisibly)
\item "extract": Character string with extracted text from file
}
}
\description{
Retrieves and displays the change log for CCSR versions. The change log
documents updates, additions, and modifications to CCSR categories across
different versions.
}
\details{
CCSR change logs document:
\itemize{
\item New CCSR categories added
\item Categories that were removed or merged
\item Changes to category descriptions
\item Updates to ICD-10 code mappings
\item Version-specific notes and improvements
}

Change logs are typically available as PDF or text documents on the HCUP
website. This function attempts to locate and retrieve them.
}
\examples{
\donttest{
# Get latest change log URL
changelog_url <- ccsr_changelog(format = "url")

# Get change log information
changelog_info <- ccsr_changelog(version = "v2026.1", format = "text")

# Download change log file
changelog_file <- ccsr_changelog(version = "v2025.1", format = "download")

# View change log in default PDF viewer
ccsr_changelog(version = "v2026.1", format = "view")

# Extract text from change log PDF (requires pdftools package)
changelog_text <- ccsr_changelog(version = "v2026.1", format = "extract")
cat(changelog_text)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/list_ccsr_versions.R
\name{list_ccsr_versions}
\alias{list_ccsr_versions}
\title{List Available CCSR Versions}
\usage{
list_ccsr_versions(type = "all")
}
\arguments{
\item{type}{Character string specifying the type of CCSR file. Must be one
of: "diagnosis" (or "dx"), "procedure" (or "pr"), or "all" (default) to
list versions for both types.}
}
\value{
A data frame (tibble) with columns:
\itemize{
\item \code{type}: The CCSR type ("diagnosis" or "procedure")
\item \code{version}: The version identifier (e.g., "v2026.1")
}
}
\description{
Returns a list of available CCSR versions for download by scraping the HCUP
website. This function helps users identify which versions are available for
diagnosis and procedure mapping files.
}
\details{
This function fetches available CCSR versions from the HCUP website.
Results are cached for 24 hours to minimize website requests. If the website
cannot be accessed, the function will return an error.
}
\examples{
\donttest{
# List all available versions
list_ccsr_versions()

# List only diagnosis versions
list_ccsr_versions("diagnosis")

# List only procedure versions
list_ccsr_versions("procedure")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_trend_table.R
\name{list_trend_table_sheets}
\alias{list_trend_table_sheets}
\title{List Available Sheets in Trend Table}
\usage{
list_trend_table_sheets(file_path)
}
\arguments{
\item{file_path}{Character string, path to a trend table Excel file (.xlsx).}
}
\value{
A character vector of sheet names.
}
\description{
Lists all available sheets in a HCUP Summary Trend Table Excel file.
}
\examples{
\donttest{
# Requires network: download first, then list sheets from that file path
path_xlsx <- download_trend_tables("2a")
list_trend_table_sheets(path_xlsx)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_ccsr.R
\name{read_ccsr}
\alias{read_ccsr}
\title{Read CCSR Mapping Files from Disk}
\usage{
read_ccsr(
  file_path = NULL,
  type = NULL,
  version = "latest",
  clean_names = TRUE,
  as_data_table = NULL,
  name = NULL
)
}
\arguments{
\item{file_path}{Optional character string, path to a CCSR mapping file. Can be:
\itemize{
\item A ZIP file path (will be extracted and read)
\item A CSV/Excel file path (will be read directly)
\item A directory path containing extracted CCSR files
If NULL (default), automatically searches the cache directory for files
downloaded via \code{download_ccsr()}.
}}

\item{type}{Character string specifying the type of CCSR file. Must be one
of: "diagnosis" (or "dx") for ICD-10-CM diagnosis codes, or "procedure"
(or "pr") for ICD-10-PCS procedure codes. If NULL and \code{file_path} is NULL,
defaults to "diagnosis". If \code{file_path} is provided, the function will
attempt to infer the type from the file name or contents.}

\item{version}{Character string specifying the CCSR version to read from cache.
Use "latest" (default) to read the most recent version, or specify a version
like "v2026.1", "v2025.1", etc. Only used when \code{file_path} is NULL.}

\item{clean_names}{Logical. If TRUE (default), column names are cleaned to
follow R naming conventions (snake_case).}

\item{as_data_table}{Logical or NULL. If TRUE and the \code{data.table} package is
available, returns a \code{data.table} object instead of a tibble. If FALSE,
returns a tibble. If NULL (default), prompts the user interactively to
choose (only in interactive sessions). In non-interactive sessions, defaults
to FALSE. Note: tibbles are already data frames and work with all standard
R data frame operations.}

\item{name}{Optional character string, suggested variable name for the
returned data. This is only used for display/messaging purposes and does
not automatically assign the data to a variable. You must still assign the
result: \code{my_data <- read_ccsr()}. If NULL (default), a name is suggested
based on the file type and version.}
}
\value{
A tibble (or data.table if \code{as_data_table = TRUE}) containing the
CCSR mapping data. Tibbles are data frames and can be used with all
standard R data frame operations, including \code{dplyr}, \code{data.table}, and
base R functions.
}
\description{
Reads previously downloaded CCSR mapping files from disk. If no file path is
provided, automatically finds and reads cached files from \code{download_ccsr()}.
}
\details{
This function can read CCSR files in several formats:
\itemize{
\item ZIP files downloaded from HCUP (will extract and read the CSV/Excel file)
\item CSV files (extracted from ZIP or saved separately)
\item Excel files (if \code{readxl} package is available)
\item Directories containing extracted files
\item Cached files from \code{download_ccsr()} (automatic if \code{file_path} is NULL)
}

The function automatically detects the file format and handles encoding
issues, preserving leading zeros in ICD-10 codes.

When \code{file_path} is NULL, the function automatically searches the cache
directory (\verb{tempdir()/HCUPtools_cache/}) for files matching the specified
\code{type} and \code{version}. This makes it easy to read previously downloaded
files without needing to know the exact file path.
}
\note{
To use the data, assign it to a variable:
\code{my_data <- read_ccsr()}. The \code{name} parameter is only for display
purposes and does not automatically assign the data.
}
\examples{
\donttest{
# Populate cache, then read (requires network)
invisible(download_ccsr("diagnosis"))
dx_map <- read_ccsr(as_data_table = FALSE)
invisible(download_ccsr("procedure"))
pr_map <- read_ccsr(type = "procedure", as_data_table = FALSE)

# From a local file on your machine (uncomment and set the path):
# dx_map <- read_ccsr("/path/to/DXCCSR-v2026-1.zip", as_data_table = FALSE)
# dx_map <- read_ccsr("/path/to/DXCCSR_v2026_1.csv", as_data_table = FALSE)
# dx_map <- read_ccsr("/path/to/extracted_ccsr_files/", as_data_table = FALSE)

head(dx_map)
nrow(dx_map)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_trend_table.R
\name{read_trend_table}
\alias{read_trend_table}
\title{Read HCUP Summary Trend Table from Disk}
\usage{
read_trend_table(
  file_path = NULL,
  table_id = NULL,
  sheet = NULL,
  clean_names = TRUE,
  as_data_table = NULL,
  name = NULL
)
}
\arguments{
\item{file_path}{Optional character string, path to a trend table Excel file (.xlsx).
If NULL (default), automatically searches the cache directory for files
downloaded via \code{download_trend_tables()} and shows an interactive menu.}

\item{table_id}{Optional character string, table ID (e.g., "1", "2a", "2b") to
read from cache. Only used when \code{file_path} is NULL. If both are NULL, shows
interactive menu.}

\item{sheet}{Character string or integer specifying which sheet to read. If
NULL (default), shows an interactive menu to select a sheet (in interactive sessions),
or automatically selects the "National" sheet (or first data sheet) in non-interactive sessions.
Common sheet names include "National", "Regional", "State", etc.}

\item{clean_names}{Logical. If TRUE (default), column names are cleaned to
follow R naming conventions (snake_case).}

\item{as_data_table}{Logical or NULL. If TRUE and the \code{data.table} package is
available, returns a \code{data.table} object instead of a tibble. If FALSE,
returns a tibble. If NULL (default), prompts the user interactively to
choose (only in interactive sessions). In non-interactive sessions, defaults
to FALSE. Note: tibbles are already data frames and work with all standard
R data frame operations.}

\item{name}{Optional character string, suggested variable name for the
returned data. This is only used for display/messaging purposes and does
not automatically assign the data to a variable. You must still assign the
result: \code{my_data <- read_trend_table()}. If NULL (default), a name is
suggested based on the table ID and sheet.}
}
\value{
A tibble (or data.table if \code{as_data_table = TRUE}) containing the
trend table data. Tibbles are data frames and can be used with all
standard R data frame operations, including \code{dplyr}, \code{data.table}, and
base R functions.
}
\description{
Reads a previously downloaded HCUP Summary Trend Table Excel file from disk.
If no file path is provided, automatically finds and reads cached files from
\code{download_trend_tables()}, with an interactive menu to select from available
tables.
}
\details{
HCUP Summary Trend Tables are Excel files with multiple sheets containing
data at different geographic levels (National, Regional, State). Use the
\code{sheet} parameter to specify which sheet to read, or call the function
multiple times with different sheets.

When \code{file_path} is NULL, the function automatically searches the cache
directory (\code{tempdir()}) for files matching the pattern \code{HCUP_SummaryTrendTables_*.xlsx}.
If multiple files are found, an interactive menu is displayed for selection.

To see available sheets, use \code{list_trend_table_sheets()}.
}
\note{
To use the data, assign it to a variable:
\code{my_data <- read_trend_table()}. The \code{name} parameter is only for display
purposes and does not automatically assign the data.
}
\examples{
\donttest{
# Requires network: download a table, list sheets, read data (same file path)
path_xlsx <- download_trend_tables("2a")
list_trend_table_sheets(path_xlsx)
national_data <- read_trend_table(file_path = path_xlsx, as_data_table = FALSE)
head(national_data)

# After a download, you can also read from cache by table ID
table_2a <- read_trend_table(table_id = "2a", as_data_table = FALSE)
head(table_2a)

# With a file already on disk, pass its path to `read_trend_table(file_path = ...)`.
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HCUPtools-package.R
\docType{package}
\name{HCUPtools-package}
\alias{HCUPtools}
\alias{HCUPtools-package}
\title{HCUPtools: Access and Work with HCUP Resources and Datasets}
\description{
\code{HCUPtools} is a comprehensive R package for accessing and working with
publicly available resources from the Agency for Healthcare Research and
Quality (AHRQ) Healthcare Cost and Utilization Project (HCUP). The package
provides streamlined access to HCUP's Clinical Classifications Software
Refined (CCSR) mapping files and Summary Trend Tables, enabling researchers
and analysts to efficiently map ICD-10 codes to CCSR categories and access
HCUP statistical reports.
}
\details{
The package provides functions to:
\itemize{
\item Download CCSR mapping files directly from the HCUP website
\item Map ICD-10-CM diagnosis codes and ICD-10-PCS procedure codes to CCSR categories
\item Access CCSR category descriptions and metadata
\item Download HCUP Summary Trend Tables
\item Read downloaded files from disk (ZIP, CSV, Excel, or directories)
\item Manage multiple CCSR versions and change logs
\item Generate proper AHRQ/HCUP citations
}

The package does \strong{not} redistribute CCSR data files but facilitates
direct download from the official AHRQ HCUP website, ensuring users
always have access to the latest versions and maintain compliance with
HCUP data use policies.

\strong{Important:} This package only accesses publicly available and free HCUP
resources (CCSR tools and Summary Trend Tables). It does NOT access any
HCUP databases (NIS, KID, SID, NEDS, etc.) that require purchase through
the HCUP Central Distributor.

For more information about CCSR, see the
\href{https://hcup-us.ahrq.gov/toolssoftware/ccsr/ccs_refined.jsp}{official HCUP CCSR overview page}.
}
\section{Main Functions}{


\strong{CCSR Mapping Functions:}
\itemize{
\item \code{download_ccsr()} - Download CCSR mapping files from HCUP website
\item \code{read_ccsr()} - Read CCSR mapping files from disk (ZIP, CSV, Excel, or directory)
\item \code{ccsr_map()} - Map ICD-10 codes to CCSR categories (long/wide/default formats)
\item \code{get_ccsr_description()} - Get clinical descriptions for CCSR codes
\item \code{list_ccsr_versions()} - List available CCSR versions
\item \code{ccsr_changelog()} - Get CCSR change log for a specific version
}

\strong{HCUP Summary Trend Tables Functions:}
\itemize{
\item \code{download_trend_tables()} - Download HCUP Summary Trend Tables
\item \code{read_trend_table()} - Read HCUP Summary Trend Table Excel files from disk
\item \code{list_trend_table_sheets()} - List available sheets in a trend table file
}

\strong{Utility Functions:}
\itemize{
\item \code{hcup_citation()} - Generate citations for HCUP resources (CCSR or Trend Tables)
}
}

\section{Key Features}{

\itemize{
\item \strong{Direct Download}: Automatically download CCSR mapping files and Summary Trend Tables from HCUP
\item \strong{Multiple Formats}: Support for long, wide, and default-only output formats
\item \strong{Cross-Classification}: Handle one-to-many mappings (multiple CCSR categories per ICD-10 code)
\item \strong{Version Management}: Access multiple CCSR versions and change logs
\item \strong{Citation Generation}: Automatically generate proper AHRQ/HCUP citations
\item \strong{File Reading}: Read downloaded files from disk (ZIP, CSV, Excel, or directories)
\item \strong{Caching}: Intelligent caching to avoid redundant downloads
\item \strong{Interactive Menus}: User-friendly interactive selection for files and options
}
}

\section{Legal and Usage}{

\strong{Important Disclaimer:} This package is an independent, non-commercial tool
developed by a third party. It is \strong{not affiliated with, endorsed by, or
supported by AHRQ or HCUP in any way.} This package is not an official AHRQ
or HCUP product. Users are responsible for ensuring compliance with all
applicable HCUP Data Use Agreements (DUAs).

\strong{User Responsibilities:}
\itemize{
\item Understanding and complying with all applicable HCUP data usage policies
\item Verifying the accuracy of results
\item Citing the appropriate AHRQ/HCUP sources in publications
\item Ensuring compliance with all HCUP Data Use Agreements
}

For official HCUP information and policies, visit:
\itemize{
\item \href{https://hcup-us.ahrq.gov/tech_assist/dua.jsp}{HCUP Data Use Agreement Training}
\item \href{https://hcup-us.ahrq.gov/team/NationwideDUA.pdf}{HCUP Data Use Agreements}
\item \href{https://hcup-us.ahrq.gov/db/publishing.jsp}{HCUP Publishing Requirements}
\item \href{https://hcup-us.ahrq.gov/toolssoftware/ccsr/ccs_refined.jsp}{CCSR Overview}
\item \href{https://hcup-us.ahrq.gov/reports/trendtables/summarytrendtables.jsp}{HCUP Summary Trend Tables}
}
}

\section{Technical Details}{

\itemize{
\item \strong{Encoding}: Files are read with UTF-8 encoding to handle special characters
\item \strong{Leading Zeros}: ICD-10 codes are preserved as character strings
\item \strong{Caching}: Downloaded files are cached by default to avoid redundant downloads
\item \strong{Cross-Classification}: Handles one-to-many mappings (multiple CCSR categories per ICD-10 code)
\item \strong{Version Management}: Automatic detection of latest versions and support for historical versions
}
}

\seealso{
Useful links:
\itemize{
\item \url{https://github.com/vikrant31/HCUPtools} - Package GitHub repository
\item \url{https://hcup-us.ahrq.gov/toolssoftware/ccsr/ccs_refined.jsp} - CCSR Overview
\item \url{https://hcup-us.ahrq.gov/toolssoftware/ccsr/ccs_refined.jsp} - CCSR Tools and Downloads
\item \url{https://hcup-us.ahrq.gov/reports/trendtables/summarytrendtables.jsp} - Summary Trend Tables
\item \url{https://hcup-us.ahrq.gov/} - HCUP Homepage
}
}
\author{
\strong{Maintainer:} Vikrant Dev Rathore \email{rathore.vikrant@gmail.com}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hcup_citation.R
\name{hcup_citation}
\alias{hcup_citation}
\title{Generate Citation for HCUP Resources}
\usage{
hcup_citation(format = "text", version = "latest", resource = "ccsr")
}
\arguments{
\item{format}{Character string specifying the citation format. Must be one
of: "text" (default), "bibtex", or "r" (for R citation object).}

\item{version}{Character string specifying the CCSR version to cite. If
"latest" (default), the function will attempt to fetch the latest version
from the HCUP website. Otherwise, specify a version like "v2026.1".}

\item{resource}{Character string specifying which HCUP resource to cite.
Options: "ccsr" (default) for CCSR data, or "trend_tables" for Summary
Trend Tables.}
}
\value{
If \code{format} is "text", returns a character string with the citation.
If \code{format} is "bibtex", returns a character string with BibTeX format.
If \code{format} is "r", returns an R citation object.
}
\description{
Provides recommended citations for HCUP resources including Clinical
Classifications Software Refined (CCSR) data and Summary Trend Tables from
the Agency for Healthcare Research and Quality (AHRQ) Healthcare Cost and
Utilization Project (HCUP).
}
\details{
This function generates citations for HCUP resources following AHRQ/HCUP
guidelines. The citation includes the appropriate version number and
access date. For CCSR data, the version is automatically detected if not
specified. For Summary Trend Tables, the citation references the general
HCUP Summary Trend Tables resource.
}
\examples{
# Text citation for CCSR
hcup_citation()

# BibTeX format for CCSR
hcup_citation(format = "bibtex")

# Citation for Summary Trend Tables
hcup_citation(resource = "trend_tables")

# R citation object
hcup_citation(format = "r")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/download_trend_tables.R
\name{download_trend_tables}
\alias{download_trend_tables}
\title{Download HCUP Summary Trend Tables}
\usage{
download_trend_tables(table_id = NULL, dest_dir = NULL, cache = TRUE)
}
\arguments{
\item{table_id}{Character string or numeric specifying which table to download.
Can be:
\itemize{
\item A table number (e.g., "1", "2a", "6b", "11c") for specific tables
\item "all" to download all available tables as a ZIP file
\item NULL (default) to show an interactive menu for selecting a table (in interactive sessions),
or return a list of available tables (in non-interactive sessions)
}}

\item{dest_dir}{Character string specifying the destination directory for the
downloaded file(s). If NULL (default), files are saved to a temporary directory.}

\item{cache}{Logical. If TRUE (default), downloaded files are cached to avoid
re-downloading on subsequent calls.}
}
\value{
If \code{table_id} is NULL and session is non-interactive, returns a data frame listing available tables.
Otherwise, returns the path(s) to the downloaded file(s).
}
\description{
Downloads HCUP Summary Trend Tables from the HCUP website. These tables provide
information on hospital utilization derived from HCUP databases, including trends
in inpatient and emergency department utilization.
}
\details{
The HCUP Summary Trend Tables include information on:
\itemize{
\item Overview of trends in inpatient and emergency department utilization
\item All inpatient encounter types
\item Inpatient encounter types (normal newborns, deliveries, elective/non-elective stays)
\item Inpatient service lines (maternal/neonatal, mental health, injuries, surgeries, etc.)
\item ED treat-and-release visits
}

Each table is available as an Excel file with state-specific, region-specific,
and national statistics.

The function automatically discovers available tables by scraping the HCUP website,
so it will automatically adapt to new tables or version changes.

For more information, see:
https://hcup-us.ahrq.gov/reports/trendtables/summarytrendtables.jsp
}
\examples{
\donttest{
# List available tables
available_tables <- download_trend_tables()
print(available_tables)

# Download a specific table
table_path <- download_trend_tables("2a")

# Bulk ZIP (\code{table_id = "all"}) is only available when HCUP publishes a
# combined file; otherwise use individual IDs from the listing above.
}

}
